@@ -30,7 +30,7 @@
 #
 
 set ::converter ""
-set ::objCmdProc "(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST* objv)"
+set ::objCmdProc "(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *const* objv)"
 
 proc convertername {type typename} {
   if {[info exists ::registeredConverter($type)]} {
@@ -42,25 +42,43 @@ proc convertername {type typename} {
   return $name
 }
 
-proc createconverter {type typename} {
+proc createconverter {type typename isGlobal} {
   set name [convertername $type $typename]
   if {[info exists ::createdConverter($name)]} {
     return ""
   }
   set domain [split $type |]
-  set opts "static const char *opts\[\] = {\"[join $domain {", "}]\", NULL};"
+
+  set localOpts [set globalOpts ""]
+  
+  if {!$isGlobal} {
+    set storageClass "static "
+    set optsName "opts"
+    set scope "localOpts"
+  } else {
+    set storageClass ""
+    set optsName "Nsf_${name}"
+    set scope "globalOpts"
+  }
+  set $scope "${storageClass}const char *$optsName\[\] = {\"[join $domain {", "}]\", NULL};"
   set ::createdConverter($name) "ConvertTo${name}, \"$type\""
+
   set enums [list ${name}NULL]
   foreach d $domain {lappend enums $name[string totitle [string map [list - _] $d]]Idx}
-  subst {
-typedef enum {[join $enums {, }]} ${name}Idx_t;
+  set enums [join $enums {, }]
 
-static int ConvertTo${name}(Tcl_Interp *interp, Tcl_Obj *objPtr, Nsf_Param const *pPtr,
+  set bindings [list @enums@ $enums @name@ $name @globalOpts@ $globalOpts \
+                    @localOpts@ $localOpts @optsName@ $optsName @typeName@ $typename]
+  
+  string map $bindings {
+typedef enum {@enums@} @name@Idx_t;
+@globalOpts@
+static int ConvertTo@name@(Tcl_Interp *interp, Tcl_Obj *objPtr, Nsf_Param const *pPtr,
 			    ClientData *clientData, Tcl_Obj **outObjPtr) {
   int index, result;
-  $opts
+  @localOpts@
   (void)pPtr;
-  result = Tcl_GetIndexFromObj(interp, objPtr, opts, "$typename", 0, &index);
+  result = Tcl_GetIndexFromObj(interp, objPtr, @optsName@, "@typeName@", 0, &index);
   *clientData = (ClientData) INT2PTR(index + 1);
   *outObjPtr = objPtr;
   return result;
@@ -79,6 +97,7 @@ proc genifd {parameterDefinitions} {
   foreach parameterDefinition $parameterDefinitions {
     array unset ""
     array set "" {-flags 0}
+    array set "" {-global 0}
     array set "" $parameterDefinition
     switch $(-type) {
       "" {set type NULL}
@@ -104,7 +123,7 @@ proc genifd {parameterDefinitions} {
       *|* {
 	if {![info exists (-typeName)]} {set (-typeName) $(-argName)}
         set converter [convertername $type $(-typeName)]
-        append ::converter [createconverter $type $(-typeName)]
+        append ::converter [createconverter $type $(-typeName) $(-global)]
         addFlags flags "NSF_ARG_IS_ENUMERATION"
       }
       default {
@@ -138,6 +157,22 @@ proc genifd {parameterDefinitions} {
   }
 }
 
+proc varName {type name} {
+  if {$type eq "tclobj" && $name ne "obj"} {
+    set varName ${name}Obj
+  } elseif {$type eq "object" && $name ne "object"} {
+    regsub -all _object $name "" name
+    set varName ${name}Object
+  } elseif {$type eq "class" && $name ne "class"} {
+    regsub -all _class $name "" name
+    set varName ${name}Class
+  } else {
+    set varName $name
+  }
+  return $varName
+}
+
+
 proc gencall {methodName fn parameterDefinitions clientData
 	      cDefsVar ifDefVar arglistVar preVar postVar introVar nnVar cleanupVar
 	    } {
@@ -150,23 +185,25 @@ proc gencall {methodName fn parameterDefinitions clientData
 
   switch $clientData {
     class {
-      set a  [list cl]
-      set if [list "NsfClass *cl"]
+      set a  [list class]
+      set if [list "NsfClass *class"]
       set argNum 3
       append intro \
-          "  NsfClass *cl;" \n\n \
+          "  NsfClass *class;" \n\n \
           "  assert(objc > 0);" \n\n \
-          "  cl = NsfObjectToClass(clientData);" \n \
-          "  if (unlikely(cl == NULL)) return NsfDispatchClientDataError(interp, clientData, \"class\", ObjStr(objv\[0\]));"
+          "  class = NsfObjectToClass(clientData);" \n \
+          "  if (unlikely(class == NULL)) {" \n \
+          "      return NsfDispatchClientDataError(interp, clientData, \"class\", ObjStr(objv\[0\]));" \n \
+          "  }" \n
     }
     object {
-      set a  [list obj]
-      set if [list "NsfObject *obj"]
+      set a  [list object]
+      set if [list "NsfObject *object"]
       set argNum 3
       append intro \
-          "  NsfObject *obj;" \n\n \
+          "  NsfObject *object;" \n\n \
           "  assert(objc > 0);" \n\n \
-          "  obj = (NsfObject *)clientData;"
+          "  object = (NsfObject *)clientData;"
     }
     ""    {
       append intro "  (void)clientData;\n"
@@ -185,7 +222,10 @@ proc gencall {methodName fn parameterDefinitions clientData
       #
       # non positional args
       #
-      set varName with[string totitle $switchName]
+      set varName [varName $(-type) $switchName]
+      if {$varName eq $switchName} {
+        set varName with[string totitle $switchName]
+      }
       set calledArg $varName
       set type "int "
       if {$(-nrargs) == 1} {
@@ -209,7 +249,7 @@ proc gencall {methodName fn parameterDefinitions clientData
       #
       # positionals
       #
-      set varName $(-argName)
+      set varName [varName $(-type) $(-argName)]
       set calledArg $varName
       switch -glob $(-type) {
         ""           {set type "const char *"}
@@ -222,15 +262,15 @@ proc gencall {methodName fn parameterDefinitions clientData
         "virtualclassargs" -
         "args"       {
           set type "int "
-          set calledArg "objc-pc.lastObjc, objv+pc.lastObjc"
-          lappend if "int nobjc" "Tcl_Obj *CONST* nobjv"
+          set calledArg "objc - pc.lastObjc, objv + pc.lastObjc"
+          lappend if "int trailingObjc" "Tcl_Obj *const* trailingObjv"
           set ifSet 1
           set cVar 0
         }
         "allargs" {
           set type "int "
           set calledArg "objc, objv"
-          lappend if "int objc" "Tcl_Obj *CONST* objv"
+          lappend if "int objc" "Tcl_Obj *const* objv"
           set ifSet 1
           set cVar 0
         }
@@ -301,8 +341,8 @@ proc gencall {methodName fn parameterDefinitions clientData
 proc genStub {stub intro obj idx cDefs pre call post cleanup} {
   # Tiny optimization for calls without parameters;
   # ParseContextExtendObjv() is just called for procs, so no need to
-  # free non-static objvs. Actually, the api for c-methods does
-  # not allow to generate structures which have to be freed.
+  # free non-static objvs. Actually, the API for c-methods does
+  # not allow one to generate structures which have to be freed.
   # we assert this in the code.
   if {$cDefs ne ""} {
     set releasePC "ParseContextRelease(&pc);"
@@ -312,7 +352,7 @@ proc genStub {stub intro obj idx cDefs pre call post cleanup} {
   }
   return [subst -nocommands {
 static int
-${stub}(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST* objv) {
+${stub}(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *const* objv) {
   ParseContext pc;
 $intro
   if (likely(ArgumentParse(interp, objc, objv, $obj, objv[0],
@@ -335,7 +375,7 @@ proc genSimpleStub {stub intro idx cDefs pre call post cleanup} {
   if {$cleanup ne ""} {error "$stub cleanup code '$cleanup' must be empty"}
   return [subst -nocommands {
 static int
-${stub}(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST* objv) {
+${stub}(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *const* objv) {
 $intro
     $cDefs
 $pre
@@ -444,8 +484,8 @@ proc genstubs {} {
     append fns [genSimpleStub $d(stub) $intro $d(idx) "" $pre $call $post $cleanup]
   } else {
       switch $d(methodType) {
-        objectMethod {set obj "obj"}
-        classMethod {set obj "(NsfObject *) cl"}
+        objectMethod {set obj "object"}
+        classMethod {set obj "(NsfObject *) class"}
         default {set obj "NULL"}
       }
       append fns [genStub $d(stub) $intro $obj $d(idx) $cDefs $pre $call $post $cleanup]
@@ -486,7 +526,8 @@ static Nsf_methodDefinition method_definitions[$nrIfds];
   puts "enum {\n $enumString\n} NsfMethods;\n"
   puts $fns
   set definitionString [join $ifds ",\n"]
-  puts "static Nsf_methodDefinition method_definitions\[$nrIfds\] = \{\n$definitionString,\{NULL\}\n\};\n"
+  set terminator {NULL, NULL, 0, {{NULL, 0, 0, NULL, NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL}}}
+  puts "static Nsf_methodDefinition method_definitions\[$nrIfds\] = \{\n$definitionString,\n{$terminator}\n\};\n"
 }
 
 proc methodDefinition {methodName methodType implementation parameterDefinitions options} {
