@@ -4,9 +4,9 @@
  *      Declarations of the internally used API Functions of the Next
  *      Scripting Framework.
  *
- *  Copyright (C) 1999-2016 Gustaf Neumann (a, b)
+ *  Copyright (C) 1999-2017 Gustaf Neumann (a, b)
  *  Copyright (C) 1999-2007 Uwe Zdun (a, b)
- *  Copyright (C) 2011-2016 Stefan Sobernig (b)
+ *  Copyright (C) 2011-2017 Stefan Sobernig (b)
  *
  * (a) University of Essen
  *     Specification of Software Systems
@@ -51,6 +51,53 @@
 # define HAVE_UINTPTR_T
 #endif
 
+/*
+ * Well behaved compiler with C99 support should define __STDC_VERSION__
+ */
+#if defined(__STDC_VERSION__)
+# if __STDC_VERSION__ >= 199901L
+#  define NSF_HAVE_C99
+# endif
+#endif
+
+/*
+ * Starting with Visual Studio 2013, Microsoft provides C99 library support.
+ */
+#if (!defined(NSF_HAVE_C99)) && defined(_MSC_VER) && (_MSC_VER >= 1800)
+# define NSF_HAVE_C99
+#endif
+
+/*
+ * Boolean type "bool" and constants
+ */
+#ifdef NSF_HAVE_C99
+   /*
+    * C99
+    */
+# include <stdbool.h>
+# define NSF_TRUE                    true
+# define NSF_FALSE                   false
+#else
+   /*
+    * Not C99
+    */
+# if defined(__cplusplus)
+   /*
+    * C++ is similar to C99, but no include necessary
+    */
+#  define NSF_TRUE                    true
+#  define NSF_FALSE                   false
+# else
+   /*
+    * If everything fails, use int type and int values for bool
+    */
+typedef int bool;
+#  define NSF_TRUE                    1
+#  define NSF_FALSE                   0
+# endif
+#endif
+
+
 /*
  * MinGW and MinGW-w64 provide both MSCRT-compliant and ANSI-compliant
  * implementations of certain I/O operations (e.g., *printf()). By
@@ -63,7 +110,7 @@
  */
 
 #if defined(__MINGW32__) && !defined(__USE_MINGW_ANSI_STDIO)
-#define __USE_MINGW_ANSI_STDIO 1
+# define __USE_MINGW_ANSI_STDIO 1
 #endif
 
 #include <tclInt.h>
@@ -100,16 +147,13 @@
 # define nonnull(ARGS)
 #endif
 
-#define nonnull_assert(assertion) assert((assertion))
-
-
 #if __GNUC_PREREQ(4, 9)
 # define returns_nonnull __attribute__((returns_nonnull))
 #else
 # define returns_nonnull
 #endif
 
-
+#define nonnull_assert(assertion) assert((assertion))
 /*
  * Tries to use gcc __attribute__ unused and mangles the name, so the
  * attribute could not be used, if declared as unused.
@@ -125,27 +169,27 @@
 
 #if defined(NSF_DTRACE)
 # include "nsfDTrace.h"
-# define NSF_DTRACE_METHOD_ENTRY_ENABLED()     		unlikely(NSF_METHOD_ENTRY_ENABLED())
-# define NSF_DTRACE_METHOD_RETURN_ENABLED()    		unlikely(NSF_METHOD_RETURN_ENABLED())
+# define NSF_DTRACE_METHOD_ENTRY_ENABLED()		unlikely(NSF_METHOD_ENTRY_ENABLED())
+# define NSF_DTRACE_METHOD_RETURN_ENABLED()		unlikely(NSF_METHOD_RETURN_ENABLED())
 # define NSF_DTRACE_OBJECT_ALLOC_ENABLED()		unlikely(NSF_OBJECT_ALLOC_ENABLED())
-# define NSF_DTRACE_OBJECT_FREE_ENABLED()  		unlikely(NSF_OBJECT_FREE_ENABLED())
-# define NSF_DTRACE_CONFIGURE_PROBE_ENABLED()  		unlikely(NSF_CONFIGURE_PROBE_ENABLED())
+# define NSF_DTRACE_OBJECT_FREE_ENABLED()		unlikely(NSF_OBJECT_FREE_ENABLED())
+# define NSF_DTRACE_CONFIGURE_PROBE_ENABLED()		unlikely(NSF_CONFIGURE_PROBE_ENABLED())
 # define NSF_DTRACE_METHOD_ENTRY(a0, a1, a2, a3, a4)	NSF_METHOD_ENTRY((a0), (a1), (a2), (a3), (a4))
-# define NSF_DTRACE_METHOD_RETURN(a0, a1, a2, a3)      	NSF_METHOD_RETURN((a0), (a1), (a2), (a3))
+# define NSF_DTRACE_METHOD_RETURN(a0, a1, a2, a3)	NSF_METHOD_RETURN((a0), (a1), (a2), (a3))
 # define NSF_DTRACE_OBJECT_ALLOC(a0, a1)		NSF_OBJECT_ALLOC((a0), (a1))
-# define NSF_DTRACE_OBJECT_FREE(a0, a1)			NSF_OBJECT_FREE((a0), (a1))
-# define NSF_DTRACE_CONFIGURE_PROBE(a0, a1)      	NSF_CONFIGURE_PROBE((a0), (a1))
+# define NSF_DTRACE_OBJECT_FREE(a0, a1)		NSF_OBJECT_FREE((a0), (a1))
+# define NSF_DTRACE_CONFIGURE_PROBE(a0, a1)		NSF_CONFIGURE_PROBE((a0), (a1))
 #else
-# define NSF_DTRACE_METHOD_ENTRY_ENABLED()     		0
-# define NSF_DTRACE_METHOD_RETURN_ENABLED()    		0
+# define NSF_DTRACE_METHOD_ENTRY_ENABLED()		0
+# define NSF_DTRACE_METHOD_RETURN_ENABLED()		0
 # define NSF_DTRACE_OBJECT_ALLOC_ENABLED()		0
-# define NSF_DTRACE_OBJECT_FREE_ENABLED()  		0
+# define NSF_DTRACE_OBJECT_FREE_ENABLED()		0
 # define NSF_DTRACE_CONFIGURE_PROBE_ENABLED()		0
-# define NSF_DTRACE_METHOD_ENTRY(a0, a1, a2, a3, a4)   	{}
-# define NSF_DTRACE_METHOD_RETURN(a0, a1, a2, a3)      	{}
+# define NSF_DTRACE_METHOD_ENTRY(a0, a1, a2, a3, a4)	{}
+# define NSF_DTRACE_METHOD_RETURN(a0, a1, a2, a3)	{}
 # define NSF_DTRACE_OBJECT_ALLOC(a0, a1)		{}
-# define NSF_DTRACE_OBJECT_FREE(a0, a1)			{}
-# define NSF_DTRACE_CONFIGURE_PROBE(a0, a1)      	{}
+# define NSF_DTRACE_OBJECT_FREE(a0, a1)		{}
+# define NSF_DTRACE_CONFIGURE_PROBE(a0, a1)		{}
 #endif
 
 
@@ -179,15 +223,14 @@ typedef struct NsfMemCounter {
 #  define MEM_COUNT_RELEASE()
 #endif
 
-# define STRING_NEW(target, p, l)  (target) = ckalloc((unsigned)(l)+1u); strncpy((target), (p), (l)); *((target)+(l)) = '\0'; \
-  MEM_COUNT_ALLOC(#target, (target))
-# define STRING_FREE(key, p)  MEM_COUNT_FREE((key), (p)); ckfree((p))
+# define STRING_NEW(target, p, l)  {char *tempValue = ckalloc((unsigned)(l)+1u); strncpy((tempValue), (p), (l)+1u); *((tempValue)+(l)) = '\0'; target = tempValue; MEM_COUNT_ALLOC(#target, (target));}
+# define STRING_FREE(key, p)  MEM_COUNT_FREE((key), (p)); ckfree((char*)(p))
 
 #define DSTRING_INIT(dsPtr) Tcl_DStringInit(dsPtr); MEM_COUNT_ALLOC("DString",(dsPtr))
 #define DSTRING_FREE(dsPtr) \
   if ((dsPtr)->string != (dsPtr)->staticSpace) {Tcl_DStringFree(dsPtr);} MEM_COUNT_FREE("DString",(dsPtr))
 
-#if USE_ASSOC_DATA
+#if defined(USE_ASSOC_DATA)
 # define RUNTIME_STATE(interp) ((NsfRuntimeState*)Tcl_GetAssocData((interp), "NsfRuntimeState", NULL))
 #else
 # define RUNTIME_STATE(interp) ((NsfRuntimeState*)((Interp*)(interp))->globalNsPtr->clientData)
@@ -216,7 +259,7 @@ typedef struct NsfMemCounter {
 	*(m)   == 'c' && (m)[1] == 'h' && (m)[2] == 'e' && (m)[3] == 'c' && \
 	(m)[4] == 'k' && (m)[5] == '\0')
 #define isCheckObjString(m) (\
-        *(m)   == 'c' && (m)[1] == 'h' && (m)[2] == 'e' && (m)[3] == 'c' && \
+	*(m)   == 'c' && (m)[1] == 'h' && (m)[2] == 'e' && (m)[3] == 'c' && \
 	(m)[4] == 'k' && (m)[5] == 'o' && (m)[6] == 'b' && (m)[7] == 'j' && \
 	(m)[8] == '\0')
 #define isCreateString(m) (\
@@ -309,6 +352,7 @@ typedef struct NsfMemCounter {
   MEM_COUNT_ALLOC("INCR_REF_COUNT-" name,(A)); Tcl_IncrRefCount((A))
 
 #define ObjStr(obj) ((obj)->bytes) ? ((obj)->bytes) : Tcl_GetString(obj)
+#define ObjTypeStr(obj) (((obj)->typePtr != NULL) ? ((obj)->typePtr->name) : "NONE")
 #define ClassName(cl) (((cl) != NULL) ? ObjStr((cl)->object.cmdName) : "NULL")
 #define ClassName_(cl) (ObjStr((cl)->object.cmdName))
 #define ObjectName(obj) (((obj) != NULL) ? ObjStr((obj)->cmdName) : "NULL")
@@ -318,8 +362,8 @@ typedef struct NsfMemCounter {
 # define PRINTOBJ(ctx,obj) \
   fprintf(stderr, "  %s %p %s oid=%p teardown=%p destroyCalled=%d\n", \
 	  (ctx),(obj),(obj)->teardown?ObjStr((obj)->cmdName):"(deleted)", \
-          (obj)->id, (obj)->teardown,                                 \
-          ((obj)->flags & NSF_DESTROY_CALLED))
+	  (obj)->id, (obj)->teardown,                                 \
+	  ((obj)->flags & NSF_DESTROY_CALLED))
 #else
 # define PRINTOBJ(ctx,obj)
 #endif
@@ -437,7 +481,7 @@ typedef struct NsfStringIncrStruct {
  * cmd flags
  */
 
-#define NSF_CMD_CALL_PROTECTED_METHOD 		0x00010000
+#define NSF_CMD_CALL_PROTECTED_METHOD		0x00010000
 #define NSF_CMD_CALL_PRIVATE_METHOD		0x00020000
 #define NSF_CMD_REDEFINE_PROTECTED_METHOD	0x00040000
 /* NSF_CMD_NONLEAF_METHOD is used to flag, if a Method implemented via cmd calls "next" */
@@ -505,7 +549,7 @@ typedef struct NsfStringIncrStruct {
 /* flags for ParseContext */
 #define NSF_PC_MUST_DECR		     0x0001u
 #define NSF_PC_IS_DEFAULT		     0x0002u
-#define NSF_PC_INVERT_DEFAULT	     	     0x0010u
+#define NSF_PC_INVERT_DEFAULT		     0x0010u
 
 #define NSF_PC_STATUS_MUST_DECR		     0x0001u
 #define NSF_PC_STATUS_FREE_OBJV		     0x0002u
@@ -543,7 +587,6 @@ typedef struct NsfParamDefs {
   int nrParams;
   int refCount;
   int serial;
-  Tcl_Obj *returns;
 } NsfParamDefs;
 
 typedef struct NsfParsedParam {
@@ -556,11 +599,11 @@ typedef struct NsfObjectOpt {
   NsfCmdList *objFilters;
   NsfCmdList *objMixins;
   ClientData clientData;
+  const char *volatileVarName;
 #if defined(PER_OBJECT_PARAMETER_CACHING)
   NsfParsedParam *parsedParamPtr;
-  int classParamPtrEpoch;
+  unsigned int classParamPtrEpoch;
 #endif
-  CONST char *volatileVarName;
   CheckOptions checkoptions;
 } NsfObjectOpt;
 
@@ -596,21 +639,21 @@ typedef struct NsfClassOpt {
 } NsfClassOpt;
 
 typedef struct NsfClass {
-  struct NsfObject object;
-  struct NsfClasses *super;
-  struct NsfClasses *sub;
+  struct NsfObject        object;
+  struct NsfClasses      *super;
+  struct NsfClasses      *sub;
   struct NsfObjectSystem *osPtr;
-  struct NsfClasses *order;
-  Tcl_HashTable instances;
-  Tcl_Namespace *nsPtr;
-  NsfParsedParam *parsedParamPtr;
-  NsfClassOpt *opt;
-  short color;
+  struct NsfClasses      *order;
+  Tcl_HashTable           instances;
+  Tcl_Namespace          *nsPtr;
+  NsfParsedParam         *parsedParamPtr;
+  NsfClassOpt            *opt;
+  short                   color;
 } NsfClass;
 
 typedef struct NsfClasses {
-  struct NsfClass *cl;
-  ClientData clientData;
+  struct NsfClass   *cl;
+  ClientData         clientData;
   struct NsfClasses *nextPtr;
 } NsfClasses;
 
@@ -621,12 +664,12 @@ typedef struct NsfClasses {
 #define NSF_PROC_FLAG_CHECK_ALWAYS 0x02u
 
 typedef struct NsfProcClientData {
-  Tcl_Obj *procName;
-  Tcl_Command cmd;
-  Tcl_Command wrapperCmd;
-  NsfParamDefs *paramDefs;
-  unsigned int flags;
+  Tcl_Obj       *procName;
+  Tcl_Command    cmd;
+  Tcl_Command    wrapperCmd;
+  NsfParamDefs  *paramDefs;
   Tcl_Namespace *origNsPtr;
+  unsigned int   flags;
 } NsfProcClientData;
 
 typedef enum SystemMethodsIdx {
@@ -648,9 +691,9 @@ typedef enum SystemMethodsIdx {
 } SystemMethodsIdx;
 
 #if !defined(NSF_C)
-EXTERN CONST char *Nsf_SystemMethodOpts[];
+EXTERN const char *Nsf_SystemMethodOpts[];
 #else
-CONST char *Nsf_SystemMethodOpts[] = {
+const char *Nsf_SystemMethodOpts[] = {
   "-class.alloc",
   "-class.create",
   "-class.dealloc",
@@ -671,14 +714,15 @@ CONST char *Nsf_SystemMethodOpts[] = {
 #endif
 
 typedef struct NsfObjectSystem {
-  NsfClass *rootClass;
-  NsfClass *rootMetaClass;
-  unsigned int overloadedMethods;
-  unsigned int definedMethods;
-  Tcl_Obj *methods[NSF_s_set_idx+2];
-  Tcl_Obj *handles[NSF_s_set_idx+2];
+  NsfClass     *rootClass;
+  NsfClass     *rootMetaClass;
+  unsigned int  overloadedMethods;
+  unsigned int  definedMethods;
+  Tcl_Obj      *methods[NSF_s_set_idx+2];
+  const char   *methodNames[NSF_s_set_idx+2];
+  Tcl_Obj      *handles[NSF_s_set_idx+2];
   struct NsfObjectSystem *nextPtr;
-  char protected[NSF_s_set_idx+2];
+  char          protected[NSF_s_set_idx+2];
 } NsfObjectSystem;
 
 
@@ -705,18 +749,18 @@ typedef enum {
   /* constants */
   NSF_ALIAS, NSF_ARGS, NSF_CMD, NSF_FILTER, NSF_FORWARD,
   NSF_METHOD,  NSF_OBJECT, NSF_SETTER, NSF_SETTERNAME, NSF_VALUECHECK,
-  NSF_GUARD_OPTION, NSF___UNKNOWN__, NSF_ARRAY, NSF_GET, NSF_SET, NSF_OPTION_STRICT,
+  NSF_GUARD_OPTION, NSF___UNKNOWN__, NSF_ARRAY, NSF_GET, NSF_SET, NSF_OPTION_STRICT, NSF_SCRIPT,
   NSF_OBJECT_UNKNOWN_HANDLER, NSF_ARGUMENT_UNKNOWN_HANDLER,
   NSF_PARSE_ARGS,
   /* Partly redefined Tcl commands; leave them together at the end */
   NSF_EXPR, NSF_FORMAT, NSF_INFO_BODY, NSF_INFO_FRAME, NSF_INTERP,
-  NSF_STRING_IS, NSF_EVAL,
-  NSF_RENAME
+  NSF_STRING_IS, NSF_EVAL, NSF_DISASSEMBLE,
+  NSF_RENAME 
 } NsfGlobalNames;
 #if !defined(NSF_C)
-EXTERN char *NsfGlobalStrings[];
+EXTERN const char *NsfGlobalStrings[];
 #else
-char *NsfGlobalStrings[] = {
+const char *NsfGlobalStrings[] = {
   "", "0", "1",
   /* methods called internally */
   "configure", "initialize", "getParameterSpec",
@@ -730,23 +774,25 @@ char *NsfGlobalStrings[] = {
   /* constants */
   "alias", "args", "cmd", "filter",  "forward",
   "method", "object", "setter", "settername", "valuecheck",
-  "-guard", "__unknown__", "::array", "get", "set", "-strict",
-  /* nsf tcl commands */
+  "-guard", "__unknown__", "::array", "get", "set", "-strict", "script",
+  /* nsf Tcl commands */
   "::nsf::object::unknown",
   "::nsf::argument::unknown",
   "::nsf::parseargs",
-  /* tcl commands */
+  /* Tcl commands */
   "expr", "format", "::tcl::info::body", "::tcl::info::frame", "interp",
-  "::tcl::string::is", "::eval",
+  "::tcl::string::is", "::eval", "::tcl::unsupported::disassemble",
   "rename"
 };
 #endif
 
 #define NsfGlobalObjs RUNTIME_STATE(interp)->methodObjNames
 
-/* obj types */
+/* 
+ * Interface for Tcl_Obj types 
+ */
 EXTERN Tcl_ObjType NsfMixinregObjType;
-EXTERN int NsfMixinregGet(Tcl_Interp *interp, Tcl_Obj *obj, NsfClass **clPtr, Tcl_Obj **guardObj)
+EXTERN int NsfMixinregGet(Tcl_Interp *interp, Tcl_Obj *obj, NsfClass **classPtr, Tcl_Obj **guardObj)
   nonnull(1) nonnull(2) nonnull(3) nonnull(4);
 EXTERN int NsfMixinregInvalidate(Tcl_Interp *interp, Tcl_Obj *obj)
   nonnull(1) nonnull(2);
@@ -755,10 +801,13 @@ EXTERN Tcl_ObjType NsfFilterregObjType;
 EXTERN int NsfFilterregGet(Tcl_Interp *interp, Tcl_Obj *obj, Tcl_Obj **filterObj, Tcl_Obj **guardObj)
   nonnull(1) nonnull(2) nonnull(3) nonnull(4);
 
-EXTERN NsfClassOpt *NsfRequireClassOpt(/*@notnull@*/ NsfClass *cl) nonnull(1) returns_nonnull;
+EXTERN NsfClassOpt *NsfRequireClassOpt(NsfClass *class)
+  nonnull(1) returns_nonnull;
 
 
-/* Next Scripting ShadowTclCommands */
+/* 
+ * Next Scripting ShadowTclCommands 
+ */
 typedef struct NsfShadowTclCommandInfo {
   TclObjCmdProcType proc;
   ClientData clientData;
@@ -771,14 +820,14 @@ typedef enum {NSF_PARAMS_NAMES, NSF_PARAMS_LIST,
 	      NSF_PARAMS_PARAMETER, NSF_PARAMS_SYNTAX} NsfParamsPrintStyle;
 
 int NsfCallCommand(Tcl_Interp *interp, NsfGlobalNames name,
-		     int objc, Tcl_Obj *CONST objv[])
+		     int objc, Tcl_Obj *const objv[])
   nonnull(1) nonnull(4);
 
 int NsfShadowTclCommands(Tcl_Interp *interp, NsfShadowOperations load)
   nonnull(1);
 
-Tcl_Obj *NsfMethodObj(NsfObject *object, int methodIdx)
-  nonnull(1);
+Tcl_Obj *NsfMethodObj(const NsfObject *object, int methodIdx)
+  nonnull(1) pure;
 
 int NsfReplaceCommandCleanup(Tcl_Interp *interp, Tcl_Obj *nameObj, NsfShadowTclCommandInfo *ti)
   nonnull(1) nonnull(2) nonnull(3);
@@ -798,15 +847,15 @@ typedef struct NsfCallStackContent {
   NsfClass *cl;
   Tcl_Command cmdPtr;
   NsfFilterStack *filterStackEntry;
-  Tcl_Obj *CONST* objv;
+  Tcl_Obj *const* objv;
   int objc;
   unsigned int flags;
-  unsigned short frameType;
 #if defined(NSF_PROFILE) || defined(NSF_DTRACE)
   long int startUsec;
   long int startSec;
-  CONST char *methodName;
+  const char *methodName;
 #endif
+  unsigned short frameType;
 } NsfCallStackContent;
 
 #define NSF_CSC_TYPE_PLAIN                    0u
@@ -852,11 +901,12 @@ typedef struct NsfCallStackContent {
 
 /*
  * Tcl uses 01 and 02, TclOO uses 04 and 08, so leave some space free
- * for further extensions of tcl and tcloo...
+ * for further extensions of Tcl and tcloo...
  */
-#define FRAME_IS_NSF_OBJECT  0x10000U
-#define FRAME_IS_NSF_METHOD  0x20000U
-#define FRAME_IS_NSF_CMETHOD 0x40000U
+#define FRAME_IS_NSF_OBJECT  0x10000u
+#define FRAME_IS_NSF_METHOD  0x20000u
+#define FRAME_IS_NSF_CMETHOD 0x40000u
+#define FRAME_VAR_LOADED     0x80000u
 
 #if defined(NRE)
 # define NRE_SANE_PATCH 1
@@ -885,9 +935,9 @@ typedef struct NsfProfile {
   Tcl_DString traceDs;
   int depth;
   int verbose;
-  int inmemory;
   Tcl_Obj *shadowedObjs;
   NsfShadowTclCommandInfo *shadowedTi;
+  int inmemory;
 } NsfProfile;
 
 # define NSF_PROFILE_TIME_DATA struct Tcl_Time profile_trt
@@ -902,6 +952,20 @@ typedef struct NsfProfile {
 # define NSF_PROFILE_EXIT(interp, object, methodName)
 #endif
 
+typedef struct NsfList {
+  void           *data;
+  Tcl_Obj        *obj;
+  struct NsfList *nextPtr;
+} NsfList;
+
+typedef struct NsfDList {
+  void  **data;
+  size_t  size;
+  size_t  avail;
+  void   *static_data[30];
+} NsfDList;
+
+
 typedef struct NsfRuntimeState {
   /*
    * The defined object systems
@@ -914,13 +978,14 @@ typedef struct NsfRuntimeState {
   Tcl_Namespace *NsfClassesNS;    /* the ::nsf::classes namespace, where classes are created physically */
   Tcl_ObjCmdProc *objInterpProc;  /* cached result of TclGetObjInterpProc() */
   Tcl_Command colonCmd;           /* cmdPtr of cmd ":" to dispatch via cmdResolver */
-  Proc fakeProc;                  /* dummy proc strucure, used for C-implemented methods with local scope */
+  Proc fakeProc;                  /* dummy proc structure, used for C-implemented methods with local scope */
   Tcl_Command currentMixinCmdPtr; /* cmdPtr of currently active mixin, used for "info activemixin" */
-  int objectMethodEpoch;
-  int instanceMethodEpoch;
+  unsigned int objectMethodEpoch;
+  unsigned int instanceMethodEpoch;
 #if defined(PER_OBJECT_PARAMETER_CACHING)
-  int classParamPtrEpoch;
+  unsigned int classParamPtrEpoch;
 #endif
+  unsigned int overloadedMethods; /* bit-array for tracking overloaded methods */
   Tcl_Obj **methodObjNames;       /* global objects of nsf */
   struct NsfShadowTclCommandInfo *tclCommands; /* shadowed Tcl commands */
 
@@ -929,10 +994,9 @@ typedef struct NsfRuntimeState {
 #endif
   int errorCount;        /* keep track of number of errors to avoid potential error loops */
   int unknown;           /* keep track whether an unknown method is currently called */
-  unsigned int overloadedMethods; /* bitarray for tracking overloaded methods */
   /*
    * Configure options. The following do*-flags could be moved into a
-   * bitarray, but we have only one state per interp, so the win on
+   * bit-array, but we have only one state per interp, so the win on
    * memory is very little.
    */
   int logSeverity;
@@ -944,16 +1008,13 @@ typedef struct NsfRuntimeState {
   int doProfile;
   int doTrace;
   unsigned int preventRecursionFlags;
-  int doClassConverterOmitUnkown;
+  int doClassConverterOmitUnknown;
   int doSoftrecreate;
-  /* keep track of defined filters */
-  Tcl_HashTable activeFilterTablePtr;
-
-  /*
-   * shutdown handling
-   */
-  int exitHandlerDestroyRound;
+  int exitHandlerDestroyRound;          /* shutdown handling */
 
+  Tcl_HashTable activeFilterTablePtr;   /* keep track of defined filters */
+  NsfList *freeListPtr;                 /* list of elements to free when interp shuts down */
+  NsfDList freeDList;
 #if defined(NSF_PROFILE)
   NsfProfile profile;
 #endif
@@ -961,9 +1022,9 @@ typedef struct NsfRuntimeState {
   void *bottomOfStack;
   void *maxStack;
 #endif
+  ClientData clientData;
   NsfStringIncrStruct iss; /* used for new to create new symbols */
   short guardCount;        /* keep track of guard invocations */
-  ClientData clientData;
 } NsfRuntimeState;
 
 #define NSF_EXITHANDLER_OFF 0
@@ -973,16 +1034,16 @@ typedef struct NsfRuntimeState {
 
 #ifdef NSF_OBJECTDATA
 EXTERN void
-NsfSetObjectData(struct NsfObject *obj, struct NsfClass *cl, ClientData data)
+NsfSetObjectData(struct NsfObject *object, struct NsfClass *class, ClientData data)
   nonnull(1) nonnull(2) nonnull(3);
 EXTERN int
-NsfGetObjectData(struct NsfObject *obj, struct NsfClass *cl, ClientData *data)
+NsfGetObjectData(struct NsfObject *object, struct NsfClass *class, ClientData *data)
   nonnull(1) nonnull(2) nonnull(3);
 EXTERN int
-NsfUnsetObjectData(struct NsfObject *obj, struct NsfClass *cl)
+NsfUnsetObjectData(struct NsfObject *object, struct NsfClass *class)
   nonnull(1) nonnull(2);
 EXTERN void
-NsfFreeObjectData(NsfClass *cl)
+NsfFreeObjectData(NsfClass *class)
   nonnull(1);
 #endif
 
@@ -1032,14 +1093,14 @@ EXTERN void NsfCleanupObject_(NsfObject *object) nonnull(1);
 
 EXTERN void NsfDeprecatedCmd(Tcl_Interp *interp, const char *what, const char *oldCmd, const char *newCmd)
   nonnull(1) nonnull(2) nonnull(3) nonnull(4);
-EXTERN void NsfProfileDeprecatedCall(Tcl_Interp *interp, NsfObject *object, NsfClass *cl,
+EXTERN void NsfProfileDeprecatedCall(Tcl_Interp *interp, NsfObject *object, NsfClass *class,
 				     const char *methodName, const char *altMethod)
   nonnull(1) nonnull(2) nonnull(4) nonnull(5);
-EXTERN void NsfProfileDebugCall(Tcl_Interp *interp, NsfObject *object, NsfClass *cl, const char *methodName,
+EXTERN void NsfProfileDebugCall(Tcl_Interp *interp, NsfObject *object, NsfClass *class, const char *methodName,
 				int objc, Tcl_Obj **objv)
   nonnull(1) nonnull(4);
-EXTERN void NsfProfileDebugExit(Tcl_Interp *interp, NsfObject *object, NsfClass *cl, const char *methodName,
-                    long startSec, long startUsec)
+EXTERN void NsfProfileDebugExit(Tcl_Interp *interp, NsfObject *object, NsfClass *class, const char *methodName,
+		    long startSec, long startUsec)
   nonnull(1) nonnull(4);
 
 #if defined(NSF_PROFILE)
@@ -1051,11 +1112,12 @@ EXTERN void NsfProfileInit(Tcl_Interp *interp) nonnull(1);
 EXTERN void NsfProfileFree(Tcl_Interp *interp) nonnull(1);
 EXTERN void NsfProfileClearData(Tcl_Interp *interp) nonnull(1);
 EXTERN void NsfProfileGetData(Tcl_Interp *interp) nonnull(1);
-EXTERN int NsfProfileTrace(Tcl_Interp *interp, int withEnable, int withVerbose, int withInmemory, Tcl_Obj *builtins);
+EXTERN int NsfProfileTrace(Tcl_Interp *interp, int withEnable, int withVerbose, int withDontsave, Tcl_Obj *builtinObjs);
 
-EXTERN void NsfProfileTraceCall(Tcl_Interp *interp, NsfObject *object, NsfClass *cl, const char *methodName)
+EXTERN void NsfProfileTraceCall(Tcl_Interp *interp, NsfObject *object, NsfClass *class, const char *methodName)
   nonnull(1) nonnull(2) nonnull(4);
-EXTERN void NsfProfileTraceExit(Tcl_Interp *interp, NsfObject *object, NsfClass *cl, const char *methodName, struct Tcl_Time *trt)
+EXTERN void NsfProfileTraceExit(Tcl_Interp *interp, NsfObject *object, NsfClass *class, const char *methodName,
+				struct Tcl_Time *callTime)
   nonnull(1) nonnull(2) nonnull(4) nonnull(5);
 EXTERN void NsfProfileTraceCallAppend(Tcl_Interp *interp, const char *label)
   nonnull(1) nonnull(2);
@@ -1066,13 +1128,12 @@ EXTERN NsfCallStackContent *NsfCallStackGetTopFrame(const Tcl_Interp *interp, Tc
   nonnull(1);
 #endif
 
-
 /*
  * MEM Counting
  */
 #ifdef NSF_MEM_COUNT
-void NsfMemCountAlloc(char *id, void *);
-void NsfMemCountFree(char *id, void *);
+void NsfMemCountAlloc(const char *id, const void *p) nonnull(1);
+void NsfMemCountFree(const char *id, const void *p) nonnull(1);
 void NsfMemCountInit(void);
 void NsfMemCountRelease(void);
 #endif /* NSF_MEM_COUNT */
@@ -1115,25 +1176,26 @@ Tcl_ObjCmdProc NsfInitProcNSCmd, NsfSelfDispatchCmd,
 
 EXTERN NsfCompEnv *NsfGetCompEnv(void);
 int NsfDirectSelfDispatch(ClientData cd, Tcl_Interp *interp,
-		     int objc, Tcl_Obj *CONST objv[])
+		     int objc, Tcl_Obj *const objv[])
   nonnull(1) nonnull(2);
 #endif
 
 EXTERN int NsfGetClassFromObj(Tcl_Interp *interp, Tcl_Obj *objPtr,
-			      NsfClass **clPtr, int withUnknown)
+			      NsfClass **classPtr, bool withUnknown)
   nonnull(1) nonnull(2) nonnull(3);
 
-EXTERN int NsfObjWrongArgs(Tcl_Interp *interp, CONST char *msg,
-			   Tcl_Obj *cmdName, Tcl_Obj *methodName,
-			   char *arglist)
+EXTERN int NsfObjWrongArgs(Tcl_Interp *interp, const char *msg,
+			   Tcl_Obj *cmdNameObj, Tcl_Obj *methodPathObj,
+			   const char *arglist)
   nonnull(1) nonnull(2);
 
-EXTERN CONST char *NsfMethodName(Tcl_Obj *methodObj)
+EXTERN const char *NsfMethodName(Tcl_Obj *methodObj)
   nonnull(1) returns_nonnull;
 
-EXTERN void NsfReportVars(Tcl_Interp *interp)
+EXTERN void NsfInitPkgConfig(Tcl_Interp *interp)
   nonnull(1);
-EXTERN void NsfDStringArgv(Tcl_DString *dsPtr, int objc, Tcl_Obj *CONST objv[])
+
+EXTERN void NsfDStringArgv(Tcl_DString *dsPtr, int objc, Tcl_Obj *const objv[])
   nonnull(1) nonnull(3);
 
 EXTERN Tcl_Obj *NsfMethodNamePath(Tcl_Interp *interp,
@@ -1171,15 +1233,19 @@ EXTERN int NsfDStringEval(Tcl_Interp *interp, Tcl_DString *dsPtr, const char *co
  * NsfFlag type
  */
 EXTERN Tcl_ObjType NsfFlagObjType;
-EXTERN int NsfFlagObjSet(Tcl_Interp *interp, Tcl_Obj *objPtr,
-			 Nsf_Param CONST *baseParamPtr, int serial,
-			 Nsf_Param CONST *paramPtr, Tcl_Obj *payload, unsigned int flags);
+EXTERN int NsfFlagObjSet(Tcl_Interp      *UNUSED(interp),
+			 Tcl_Obj         *objPtr,
+			 Nsf_Param const *baseParamPtr,
+			 int              serial,
+			 Nsf_Param const *paramPtr,
+			 Tcl_Obj         *payload,
+			 unsigned int     flags);
 typedef struct {
-  CONST Nsf_Param *signature;
-  int serial;
-  Nsf_Param CONST *paramPtr;
-  Tcl_Obj *payload;
-  unsigned int flags;
+  const Nsf_Param *signature;
+  Nsf_Param const *paramPtr;
+  Tcl_Obj         *payload;
+  int              serial;
+  unsigned int     flags;
 } NsfFlag;
 
 #define NSF_FLAG_DASHDAH		0x01
@@ -1190,20 +1256,26 @@ typedef struct {
  */
 EXTERN Tcl_ObjType NsfInstanceMethodObjType;
 EXTERN Tcl_ObjType NsfObjectMethodObjType;
-EXTERN int NsfMethodObjSet(Tcl_Interp *interp, Tcl_Obj *objPtr,
-			   Tcl_ObjType *objectType,
-			   void *context, int methodEpoch,
-			   Tcl_Command cmd, NsfClass *cl, unsigned int flags)
-  nonnull(1) nonnull(2) nonnull(3) nonnull(4) nonnull(6);
+
+EXTERN int NsfMethodObjSet(
+    Tcl_Interp  *UNUSED(interp),
+    Tcl_Obj     *objPtr,
+    const Tcl_ObjType *objectType,
+    void        *context,
+    unsigned int methodEpoch,
+    Tcl_Command  cmd,
+    NsfClass    *class,
+    unsigned int flags
+) nonnull(1) nonnull(2) nonnull(3) nonnull(4) nonnull(6);
 
 
 
 
 typedef struct {
-  void *context;
-  int methodEpoch;
-  Tcl_Command cmd;
-  NsfClass *cl;
+  void        *context;
+  Tcl_Command  cmd;
+  NsfClass    *cl;
+  unsigned int methodEpoch;
   unsigned int flags;
 } NsfMethodContext;
 
@@ -1227,7 +1299,7 @@ void NsfStringIncrFree(NsfStringIncrStruct *iss)
  *
  */
 
-typedef void (Nsf_AnyFun)();
+typedef void (Nsf_AnyFun)(void);
 
 EXTERN void Nsf_InitFunPtrHashTable(Tcl_HashTable *tablePtr)
   nonnull(1);
@@ -1240,11 +1312,8 @@ EXTERN Tcl_HashEntry *Nsf_FindFunPtrHashEntry(Tcl_HashTable *tablePtr, Nsf_AnyFu
 /*
  * NSF enumeration-type interface
  */
-EXTERN void Nsf_EnumerationTypeInit(Tcl_Interp *interp)
-  nonnull(1);
-
-EXTERN void Nsf_EnumerationTypeRelease(Tcl_Interp *interp)
-  nonnull(1);
+EXTERN void Nsf_EnumerationTypeInit(void);
+EXTERN void Nsf_EnumerationTypeRelease(void);
 
 EXTERN const char *Nsf_EnumerationTypeGetDomain(Nsf_TypeConverter *converter)
   nonnull(1);
@@ -1252,18 +1321,14 @@ EXTERN const char *Nsf_EnumerationTypeGetDomain(Nsf_TypeConverter *converter)
 /*
  * NSF command definitions interface
  */
-EXTERN void Nsf_CmdDefinitionInit(Tcl_Interp *interp)
-  nonnull(1);
-
-EXTERN void Nsf_CmdDefinitionRelease(Tcl_Interp *interp)
-  nonnull(1);
-
+EXTERN void Nsf_CmdDefinitionInit(void);
+EXTERN void Nsf_CmdDefinitionRelease(void);
 EXTERN Nsf_methodDefinition *Nsf_CmdDefinitionGet(Tcl_ObjCmdProc *proc)
   nonnull(1);
 
 
 #ifndef HAVE_STRNSTR
-char *strnstr(const char *buffer, const char *needle, size_t buffer_len);
+char *strnstr(const char *buffer, const char *needle, size_t buffer_len) pure;
 #endif
 
 /*
@@ -1272,7 +1337,7 @@ char *strnstr(const char *buffer, const char *needle, size_t buffer_len);
    versions provide a prefixed (__*) one. The by-feature test below
    falls back to the prefixed version, if available, and provides a
    more general fallback to a simple assignment; this is primarily for
-   MSVC; admittedly, this simplifcation is not generally portable to
+   MSVC; admittedly, this simplification is not generally portable to
    platform/compiler combos other then x86, but the best I can think of right
    now. One might constrain the assignment-fallback to a platform and
    leave va_copy undefined in uncaught platform cases (?).
@@ -1292,4 +1357,15 @@ char *strnstr(const char *buffer, const char *needle, size_t buffer_len);
 #define vsnprintf _vsnprintf
 #endif
 
+/* 
+ * There are six whitespace characters in Tcl, which serve as element
+ * separators in string representations of Tcl lists. See tclUtil.c
+ */
+
+#define NsfHasTclSpace(str) \
+  (strpbrk((str), " \t\n\r\v\f") != NULL)
+
+#define NsfMax(a,b) ((a) > (b) ? a : b)
+#define NsfMin(a,b) ((a) < (b) ? a : b)
+
 #endif /* _nsf_int_h_ */
