@@ -3,8 +3,8 @@
  *
  *      Stack handling functions of the Next Scripting Framework.
  *
- * Copyright (C) 2010-2016 Gustaf Neumann
- * Copyright (C) 2011-2016 Stefan Sobernig
+ * Copyright (C) 2010-2017 Gustaf Neumann
+ * Copyright (C) 2011-2017 Stefan Sobernig
  *
  * Vienna University of Economics and Business
  * Institute of Information Systems and New Media
@@ -33,6 +33,135 @@
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
  */
+
+/*
+ * Static functions defined in this file.
+ */
+static void Nsf_PushFrameObj(
+    Tcl_Interp *interp,
+    NsfObject *object,
+    const CallFrame *framePtr
+) nonnull(1) nonnull(2) nonnull(3);
+
+static void Nsf_PopFrameObj(Tcl_Interp *interp, CallFrame *framePtr)
+  nonnull(1) nonnull(2);
+
+NSF_INLINE static void Nsf_PushFrameCsc(
+    Tcl_Interp *interp,
+    const NsfCallStackContent *cscPtr,
+    CallFrame *framePtr
+) nonnull(1) nonnull(2) nonnull(3);
+
+static void Nsf_PopFrameCsc(Tcl_Interp *interp, CallFrame *UNUSED(framePtr))
+  nonnull(1);
+
+static Tcl_CallFrame* CallStackGetActiveProcFrame(Tcl_CallFrame *framePtr)
+  nonnull(1) pure;
+
+NSF_INLINE static NsfObject* GetSelfObj2(
+    const Tcl_Interp *UNUSED(interp),
+    Tcl_CallFrame *framePtr
+) nonnull(2) pure;
+
+static Tcl_CallFrame* CallStackGetTclFrame(
+    const Tcl_Interp *interp,
+    Tcl_CallFrame *varFramePtr,
+    int skip
+) nonnull(1) pure;
+
+static NsfCallStackContent* CallStackGetTopFrame(
+    const Tcl_Interp *interp,
+    Tcl_CallFrame **framePtrPtr
+) nonnull(1);
+
+NSF_INLINE static NsfCallStackContent* CallStackGetTopFrame0(const Tcl_Interp *interp)
+  nonnull(1) pure;
+
+static NsfCallStackContent* NsfCallStackFindLastInvocation(
+    const Tcl_Interp *interp,
+    int offset,
+    Tcl_CallFrame **framePtrPtr
+) nonnull(1);
+
+static NsfCallStackContent* NsfCallStackFindActiveFrame(
+    const Tcl_Interp *interp,
+    int offset,
+    Tcl_CallFrame **framePtrPtr
+) nonnull(1);
+
+static void CallStackUseActiveFrame(
+    const Tcl_Interp *interp,
+    callFrameContext *ctx
+) nonnull(1) nonnull(2);
+
+static void CallStackRestoreSavedFrames(
+    Tcl_Interp *interp,
+    const callFrameContext *ctx
+) nonnull(1) nonnull(2);
+
+static NsfCallStackContent* CallStackFindActiveFilter(const Tcl_Interp *interp)
+  nonnull(1) pure;
+
+static NsfCallStackContent* CallStackFindEnsembleCsc(
+    const Tcl_CallFrame *framePtr,
+    Tcl_CallFrame **framePtrPtr
+) nonnull(1) nonnull(2);
+
+
+static Tcl_CallFrame* CallStackNextFrameOfType(
+    Tcl_CallFrame *framePtr,
+    unsigned int flags
+) nonnull(1);
+
+static Tcl_Obj* CallStackMethodPath(Tcl_Interp *interp, Tcl_CallFrame *framePtr)
+  nonnull(1) nonnull(2);
+
+NSF_INLINE static int FilterActiveOnObj(
+    const Tcl_Interp *interp,
+    const NsfObject *object,
+    const Tcl_Command cmd
+) nonnull(1) nonnull(2) pure;
+
+static void CallStackReplaceVarTableReferences(
+    const Tcl_Interp *interp,
+    TclVarHashTable *oldVarTablePtr,
+    TclVarHashTable *newVarTablePtr
+) nonnull(1) nonnull(2) nonnull(3);
+
+#if defined(NRE)
+static NsfCallStackContent* CscAlloc(
+    Tcl_Interp *interp,
+    NsfCallStackContent *cscPtr,
+    const Tcl_Command cmd
+) nonnull(1);
+#else
+static NsfCallStackContent* CscAlloc(
+    Tcl_Interp *interp,
+    NsfCallStackContent *cscPtr,
+    const Tcl_Command cmd
+) nonnull(2);
+#endif
+
+NSF_INLINE static void CscInit_(
+    NsfCallStackContent *cscPtr,
+    NsfObject *object,
+    NsfClass *class,
+    const Tcl_Command cmd,
+    unsigned short frameType,
+    unsigned int flags
+) nonnull(1) nonnull(2);
+
+NSF_INLINE static void CscFinish_(Tcl_Interp *interp, NsfCallStackContent *cscPtr)
+  nonnull(1) nonnull(2);
+
+
+
+
+
+
+
+
+
 #ifdef CHECK_ACTIVATION_COUNTS
 static NsfClasses * NsfClassListUnlink(NsfClasses **firstPtrPtr, const void *key);
 
@@ -52,7 +181,8 @@ static NsfClasses * NsfClassListUnlink(NsfClasses **firstPtrPtr, const void *key
  */
 static int CscListRemove(const Tcl_Interp *interp, const NsfCallStackContent *cscPtr, NsfClasses **cscListPtr)
   nonnull(1) nonnull(2);
-static void CscListAdd(const Tcl_Interp *interp, const NsfCallStackContent *cscPtr) nonnull(1) nonnull(2);
+static void CscListAdd(const Tcl_Interp *interp, const NsfCallStackContent *cscPtr)
+  nonnull(1) nonnull(2);
 
 static void
 CscListAdd(const Tcl_Interp *interp, const NsfCallStackContent *cscPtr) {
@@ -101,7 +231,7 @@ CscListRemove(const Tcl_Interp *interp, const NsfCallStackContent *cscPtr, NsfCl
  *----------------------------------------------------------------------
  * NsfShowStack --
  *
- *    Print the contents of the callstack to stderr. This function is
+ *    Print the contents of the call-stack to stderr. This function is
  *    for debugging purposes only.
  *
  * Results:
@@ -115,6 +245,8 @@ CscListRemove(const Tcl_Interp *interp, const NsfCallStackContent *cscPtr, NsfCl
 void NsfShowStack(Tcl_Interp *interp) {
   Tcl_CallFrame *framePtr;
 
+  nonnull_assert(interp != NULL);
+
   fprintf(stderr, "NsfShowStack framePtr %p varFramePtr %p\n",
           (void *)Tcl_Interp_framePtr(interp), (void *)Tcl_Interp_varFramePtr(interp));
   /* framePtr = (Tcl_CallFrame *)Tcl_Interp_varFramePtr(interp);
@@ -141,7 +273,7 @@ void NsfShowStack(Tcl_Interp *interp) {
             Tcl_CallFrame_objc(framePtr) > 0 ? Tcl_CallFrame_objc(framePtr) : -1);
     if (cscPtr != NULL) {
       fprintf(stderr, " csc %p frameType %.4x flags %.6x (%s.%p %s)\n",
-	      (void *)cscPtr,
+              (void *)cscPtr,
               cscPtr->frameType,
               cscPtr->flags,
               ObjectName(cscPtr->self),
@@ -169,7 +301,7 @@ void NsfShowStack(Tcl_Interp *interp) {
  *----------------------------------------------------------------------
  * Nsf_PushFrameObj, Nsf_PopFrameObj --
  *
- *    Push or pop a frame with a callstack content as an OBJECT
+ *    Push or pop a frame with a call-stack content as an OBJECT
  *    frame.
  *
  * Results:
@@ -180,9 +312,6 @@ void NsfShowStack(Tcl_Interp *interp) {
  *
  *----------------------------------------------------------------------
  */
-static void Nsf_PushFrameObj(Tcl_Interp *interp, NsfObject *object, const CallFrame *framePtr) nonnull(1) nonnull(2) nonnull(3);
-static void Nsf_PopFrameObj(Tcl_Interp *interp, CallFrame *framePtr) nonnull(1) nonnull(2);
-
 static void Nsf_PushFrameObj(Tcl_Interp *interp, NsfObject *object, const CallFrame *framePtr) {
 
   nonnull_assert(interp != NULL);
@@ -224,7 +353,7 @@ static void Nsf_PopFrameObj(Tcl_Interp *interp, CallFrame *framePtr) {
  *----------------------------------------------------------------------
  * Nsf_PushFrameCsc, Nsf_PopFrameCsc --
  *
- *    Push or pop a frame with a callstack content as a CMETHOD
+ *    Push or pop a frame with a call-stack content as a CMETHOD
  *    frame.
  *
  * Results:
@@ -235,9 +364,6 @@ static void Nsf_PopFrameObj(Tcl_Interp *interp, CallFrame *framePtr) {
  *
  *----------------------------------------------------------------------
  */
-NSF_INLINE static void Nsf_PushFrameCsc(Tcl_Interp *interp, const NsfCallStackContent *cscPtr, CallFrame *framePtr)
-  nonnull(1) nonnull(2) nonnull(3);
-static void Nsf_PopFrameCsc(Tcl_Interp *interp, CallFrame *UNUSED(framePtr)) nonnull(1);
 
 NSF_INLINE static void
 Nsf_PushFrameCsc(Tcl_Interp *interp, const NsfCallStackContent *cscPtr, CallFrame *framePtr) {
@@ -251,7 +377,7 @@ Nsf_PushFrameCsc(Tcl_Interp *interp, const NsfCallStackContent *cscPtr, CallFram
     framePtr, cscPtr, Tcl_GetCommandName(interp, cscPtr->cmdPtr));*/
 
   Tcl_PushCallFrame(interp, (Tcl_CallFrame *)framePtr, Tcl_CallFrame_nsPtr(varFramePtr),
-		    FRAME_IS_PROC|FRAME_IS_NSF_CMETHOD);
+                    FRAME_IS_PROC|FRAME_IS_NSF_CMETHOD);
   Tcl_CallFrame_clientData(framePtr) = (ClientData)cscPtr;
   Tcl_CallFrame_procPtr(framePtr) = &RUNTIME_STATE(interp)->fakeProc;
 }
@@ -283,8 +409,6 @@ Nsf_PopFrameCsc(Tcl_Interp *interp, CallFrame *UNUSED(framePtr)) {
  *
  *----------------------------------------------------------------------
  */
-static Tcl_CallFrame * CallStackGetActiveProcFrame(Tcl_CallFrame *framePtr) nonnull(1);
-
 static Tcl_CallFrame *
 CallStackGetActiveProcFrame(Tcl_CallFrame *framePtr) {
 
@@ -296,7 +420,7 @@ CallStackGetActiveProcFrame(Tcl_CallFrame *framePtr) {
     if ((flag & (FRAME_IS_NSF_METHOD|FRAME_IS_NSF_CMETHOD)) != 0) {
       /* never return an inactive method frame */
       if (likely(!(((NsfCallStackContent *)Tcl_CallFrame_clientData(framePtr))->frameType
-		   & NSF_CSC_TYPE_INACTIVE))) {
+                   & NSF_CSC_TYPE_INACTIVE))) {
         break;
       }
     } else {
@@ -314,40 +438,11 @@ CallStackGetActiveProcFrame(Tcl_CallFrame *framePtr) {
 
 /*
  *----------------------------------------------------------------------
- * CallStackNextFrameOfType --
+ * GetSelfObj, GetSelfObj2 --
  *
- *    Return the next frame with a specified type from the call stack.
- *    The type is specified by a bit mask passed as flags.
- *
- * Results:
- *    Tcl_CallFrame
- *
- * Side effects:
- *    None.
- *
- *----------------------------------------------------------------------
- */
-static Tcl_CallFrame * CallStackNextFrameOfType(Tcl_CallFrame *framePtr, unsigned int flags) nonnull(1);
-
-static Tcl_CallFrame *
-CallStackNextFrameOfType(Tcl_CallFrame *framePtr, unsigned int flags) {
-  nonnull_assert(framePtr != NULL);
-
-  do {
-    if (((unsigned int)Tcl_CallFrame_isProcCallFrame(framePtr) & flags) != 0u) {
-      return framePtr;
-    }
-    framePtr = Tcl_CallFrame_callerPtr(framePtr);
-  } while (framePtr != NULL);
-
-  return framePtr;
-}
-
-/*
- *----------------------------------------------------------------------
- * GetSelfObj --
- *
- *    Return the currently active object from a method or object frame.
+ *    Return the corresponding object from a method or from an object
+ *    frame. GetSelfObj defaults to the top-most call-frame, GetSelfObj2 allows
+ *    one to set another frame.
  *
  * Results:
  *    NsfObject * or NULL.
@@ -367,37 +462,39 @@ CallStackNextFrameOfType(Tcl_CallFrame *framePtr, unsigned int flags) {
 # endif
 #endif
 
-NSF_INLINE static NsfObject* GetSelfObj(const Tcl_Interp *interp) nonnull(1);
+#define GetSelfObj(interp) \
+  GetSelfObj2((interp), (Tcl_CallFrame *)Tcl_Interp_varFramePtr((interp)))
 
 NSF_INLINE static NsfObject*
-GetSelfObj(const Tcl_Interp *interp) {
-  register Tcl_CallFrame *varFramePtr = (Tcl_CallFrame *)Tcl_Interp_varFramePtr(interp);
+GetSelfObj2(const Tcl_Interp *UNUSED(interp), Tcl_CallFrame *framePtr) {
+  register Tcl_CallFrame *varFramePtr;
 
-  nonnull_assert(interp != NULL);
+  nonnull_assert(framePtr != NULL);
 
   /*fprintf(stderr, "GetSelfObj interp has frame %p and var-frame %p\n",
     Tcl_Interp_framePtr(interp), Tcl_Interp_varFramePtr(interp));*/
 
-  for (; varFramePtr != NULL; varFramePtr =
-
+  for (varFramePtr = framePtr;
+       varFramePtr != NULL;
+       varFramePtr =
 #if defined(SKIP_LEVELS)
-			Tcl_CallFrame_callerPtr(varFramePtr)
+                        Tcl_CallFrame_callerPtr(varFramePtr)
 #else
-	 		NULL
+                        NULL
 #endif
-			) {
-    register unsigned int flags = (unsigned int)Tcl_CallFrame_isProcCallFrame(varFramePtr);
+                        ) {
+    register unsigned int flags;
 
-    if (likely((flags & (FRAME_IS_NSF_METHOD|FRAME_IS_NSF_CMETHOD)))) {
-      const NsfCallStackContent *cscPtr = (NsfCallStackContent *)Tcl_CallFrame_clientData(varFramePtr);
-      return cscPtr->self;
-
-    } else if (flags & FRAME_IS_NSF_OBJECT) {
+    flags = (unsigned int)Tcl_CallFrame_isProcCallFrame(varFramePtr);
+    if (likely((flags & (FRAME_IS_NSF_METHOD|FRAME_IS_NSF_CMETHOD)) != 0u)) {
+      return ((NsfCallStackContent *)Tcl_CallFrame_clientData(varFramePtr))->self;
 
+    } else if ((flags & FRAME_IS_NSF_OBJECT) != 0u) {
       return (NsfObject *)Tcl_CallFrame_clientData(varFramePtr);
+
     }
 #if defined(SKIP_LAMBDA)
-    if (flags & FRAME_IS_LAMBDA) {
+    if ((flags & FRAME_IS_LAMBDA) != 0u) {
       continue;
     }
     break;
@@ -410,7 +507,7 @@ GetSelfObj(const Tcl_Interp *interp) {
  *----------------------------------------------------------------------
  * CallStackGetTclFrame --
  *
- *    Return the Tcl_Callframe a (scripted or nonleaf) method starting with
+ *    Return the Tcl_Callframe a (scripted or non-leaf) method starting with
  *    the specified or topmost frame; if skip is a positive number the
  *    specified number of Tcl frames are skipped.
  *
@@ -422,13 +519,9 @@ GetSelfObj(const Tcl_Interp *interp) {
  *
  *----------------------------------------------------------------------
  */
-static  Tcl_CallFrame* CallStackGetTclFrame(const Tcl_Interp *interp,
-					    Tcl_CallFrame *startFramePtr,
-					    int skip) nonnull(1);
-
 static Tcl_CallFrame* CallStackGetTclFrame(const Tcl_Interp *interp,
-					   Tcl_CallFrame *varFramePtr,
-					   int skip) {
+                                           Tcl_CallFrame *varFramePtr,
+                                           int skip) {
   nonnull_assert(interp != NULL);
   assert(skip >= 0);
 
@@ -456,7 +549,7 @@ static Tcl_CallFrame* CallStackGetTclFrame(const Tcl_Interp *interp,
  * CallStackGetTopFrame, CallStackGetTopFrame0, NsfCallStackGetTopFrame --
  *
  *    Return the NsfCallStackContent* of the topmost invocation of a (scripted
- *    or nonleaf) method. If framePtrPtr is provided, it is used to return the
+ *    or non-leaf) method. If framePtrPtr is provided, it is used to return the
  *    Tcl frame as well.
  *
  * Results:
@@ -467,9 +560,6 @@ static Tcl_CallFrame* CallStackGetTclFrame(const Tcl_Interp *interp,
  *
  *----------------------------------------------------------------------
  */
-static NsfCallStackContent*
-CallStackGetTopFrame(const Tcl_Interp *interp, Tcl_CallFrame **framePtrPtr) nonnull(1);
-
 static NsfCallStackContent*
 CallStackGetTopFrame(const Tcl_Interp *interp, Tcl_CallFrame **framePtrPtr) {
   register Tcl_CallFrame *varFramePtr;
@@ -493,11 +583,10 @@ CallStackGetTopFrame(const Tcl_Interp *interp, Tcl_CallFrame **framePtrPtr) {
   return NULL;
 }
 
-NSF_INLINE static NsfCallStackContent* CallStackGetTopFrame0(const Tcl_Interp *interp) nonnull(1);
-
 NSF_INLINE static NsfCallStackContent*
 CallStackGetTopFrame0(const Tcl_Interp *interp) {
   register Tcl_CallFrame *varFramePtr;
+  NsfCallStackContent    *result = NULL;
 
   nonnull_assert(interp != NULL);
 
@@ -505,14 +594,16 @@ CallStackGetTopFrame0(const Tcl_Interp *interp) {
        varFramePtr != NULL;
        varFramePtr = Tcl_CallFrame_callerPtr(varFramePtr)) {
     if (likely(((unsigned int)Tcl_CallFrame_isProcCallFrame(varFramePtr) & (FRAME_IS_NSF_METHOD|FRAME_IS_NSF_CMETHOD)) != 0u)) {
-      return (NsfCallStackContent *)Tcl_CallFrame_clientData(varFramePtr);
+      result = (NsfCallStackContent *)Tcl_CallFrame_clientData(varFramePtr);
+      break;
     }
   }
-  return NULL;
+  return result;
 }
 
 #if defined(NSF_PROFILE)
-NsfCallStackContent* NsfCallStackGetTopFrame(const Tcl_Interp *interp, Tcl_CallFrame **framePtrPtr) nonnull(1);
+NsfCallStackContent* NsfCallStackGetTopFrame(const Tcl_Interp *interp, Tcl_CallFrame **framePtrPtr)
+  nonnull(1);
 
 NsfCallStackContent*
 NsfCallStackGetTopFrame(const Tcl_Interp *interp, Tcl_CallFrame **framePtrPtr) {
@@ -524,7 +615,7 @@ NsfCallStackGetTopFrame(const Tcl_Interp *interp, Tcl_CallFrame **framePtrPtr) {
  *----------------------------------------------------------------------
  * NsfCallStackFindLastInvocation --
  *
- *    Find last invocation of a (scripted or nonleaf) method with a
+ *    Find last invocation of a (scripted or non-leaf) method with a
  *    specified offset.
  *
  * Results:
@@ -535,9 +626,6 @@ NsfCallStackGetTopFrame(const Tcl_Interp *interp, Tcl_CallFrame **framePtrPtr) {
  *
  *----------------------------------------------------------------------
  */
-static NsfCallStackContent *NsfCallStackFindLastInvocation(const Tcl_Interp *interp, int offset, Tcl_CallFrame **framePtrPtr)
-  nonnull(1);
-
 static NsfCallStackContent *
 NsfCallStackFindLastInvocation(const Tcl_Interp *interp, int offset, Tcl_CallFrame **framePtrPtr) {
   register Tcl_CallFrame *varFramePtr = (Tcl_CallFrame *)Tcl_Interp_varFramePtr(interp);
@@ -554,17 +642,17 @@ NsfCallStackFindLastInvocation(const Tcl_Interp *interp, int offset, Tcl_CallFra
        * A NSF method frame.
        */
       if ((cscPtr->flags & (NSF_CSC_CALL_IS_NEXT|NSF_CSC_CALL_IS_ENSEMBLE))
-	  || (cscPtr->frameType & NSF_CSC_TYPE_INACTIVE)) {
+          || (cscPtr->frameType & NSF_CSC_TYPE_INACTIVE)) {
         continue;
       }
 
       if (offset != 0) {
         offset--;
       } else if (Tcl_CallFrame_level(varFramePtr) < lvl) {
-	if (framePtrPtr != NULL) {
+        if (framePtrPtr != NULL) {
           *framePtrPtr = varFramePtr;
         }
-	return cscPtr;
+        return cscPtr;
       }
     } else if (Tcl_CallFrame_isProcCallFrame(varFramePtr)) {
 
@@ -574,10 +662,10 @@ NsfCallStackFindLastInvocation(const Tcl_Interp *interp, int offset, Tcl_CallFra
       if (offset != 0) {
         offset--;
       } else if (Tcl_CallFrame_level(varFramePtr) < lvl) {
-	if (framePtrPtr != NULL) {
+        if (framePtrPtr != NULL) {
           *framePtrPtr = varFramePtr;
         }
-	return NULL;
+        return NULL;
       }
     }
   }
@@ -592,7 +680,7 @@ NsfCallStackFindLastInvocation(const Tcl_Interp *interp, int offset, Tcl_CallFra
  *----------------------------------------------------------------------
  * NsfCallStackFindActiveFrame --
  *
- *    Search for the first active frame on the callstack.
+ *    Search for the first active frame on the call-stack.
  *
  * Results:
  *    Call stack content or NULL.
@@ -602,8 +690,6 @@ NsfCallStackFindLastInvocation(const Tcl_Interp *interp, int offset, Tcl_CallFra
  *
  *----------------------------------------------------------------------
  */
-static NsfCallStackContent *NsfCallStackFindActiveFrame(const Tcl_Interp *interp, int offset, Tcl_CallFrame **framePtrPtr) nonnull(1);
-
 static NsfCallStackContent *
 NsfCallStackFindActiveFrame(const Tcl_Interp *interp, int offset, Tcl_CallFrame **framePtrPtr) {
   register Tcl_CallFrame *varFramePtr;
@@ -615,7 +701,7 @@ NsfCallStackFindActiveFrame(const Tcl_Interp *interp, int offset, Tcl_CallFrame
        (offset > 0) && (varFramePtr != NULL);
        varFramePtr = Tcl_CallFrame_callerPtr(varFramePtr), offset--);
 
-  /* search for first active frame and set tcl frame pointers */
+  /* search for first active frame and set Tcl frame pointers */
   for (; varFramePtr != NULL; varFramePtr = Tcl_CallFrame_callerPtr(varFramePtr)) {
     if (((unsigned int)Tcl_CallFrame_isProcCallFrame(varFramePtr) & (FRAME_IS_NSF_METHOD|FRAME_IS_NSF_CMETHOD)) != 0u) {
       NsfCallStackContent *cscPtr = (NsfCallStackContent *)Tcl_CallFrame_clientData(varFramePtr);
@@ -652,8 +738,6 @@ NsfCallStackFindActiveFrame(const Tcl_Interp *interp, int offset, Tcl_CallFrame
  *
  *----------------------------------------------------------------------
  */
-static void CallStackUseActiveFrame(const Tcl_Interp *interp, callFrameContext *ctx) nonnull(1) nonnull(2);
-
 static void
 CallStackUseActiveFrame(const Tcl_Interp *interp, callFrameContext *ctx) {
   Tcl_CallFrame *framePtr, *inFramePtr;
@@ -670,12 +754,12 @@ CallStackUseActiveFrame(const Tcl_Interp *interp, callFrameContext *ctx) {
 
   if (inFramePtr == framePtr) {
     /* call frame pointers are fine */
-    ctx->frameSaved = 0;
+    ctx->frameSaved = NSF_FALSE;
   } else {
     ctx->varFramePtr = inFramePtr;
     /*fprintf(stderr, "CallStackUseActiveFrame stores %p\n", framePtr);*/
     Tcl_Interp_varFramePtr(interp) = (CallFrame *)framePtr;
-    ctx->frameSaved = 1;
+    ctx->frameSaved = NSF_TRUE;
   }
 }
 
@@ -695,15 +779,13 @@ CallStackUseActiveFrame(const Tcl_Interp *interp, callFrameContext *ctx) {
  *
  *----------------------------------------------------------------------
  */
-static void CallStackRestoreSavedFrames(Tcl_Interp *interp, callFrameContext *ctx) nonnull(1) nonnull(2);
-
 static void
-CallStackRestoreSavedFrames(Tcl_Interp *interp, callFrameContext *ctx) {
+CallStackRestoreSavedFrames(Tcl_Interp *interp, const callFrameContext *ctx) {
 
   nonnull_assert(interp != NULL);
   nonnull_assert(ctx != NULL);
 
-  if (ctx->frameSaved != 0) {
+  if (ctx->frameSaved) {
     /*fprintf(stderr, "CallStackRestoreSavedFrames drops %p restores %p\n",
       Tcl_Interp_varFramePtr(interp), ctx->varFramePtr);*/
     Tcl_Interp_varFramePtr(interp) = (CallFrame *)ctx->varFramePtr;
@@ -714,21 +796,19 @@ CallStackRestoreSavedFrames(Tcl_Interp *interp, callFrameContext *ctx) {
  *----------------------------------------------------------------------
  * CallStackFindActiveFilter --
  *
- *    Return the callstack content of the currently active filter
+ *    Return the call-stack content of the currently active filter
  *
  * Results:
- *    Callstack content or NULL, if no filter is active
+ *    Call-stack content or NULL, if no filter is active
  *
  * Side effects:
  *    None.
  *
  *----------------------------------------------------------------------
  */
-static NsfCallStackContent * CallStackFindActiveFilter(const Tcl_Interp *interp) nonnull(1);
-
 static NsfCallStackContent *
 CallStackFindActiveFilter(const Tcl_Interp *interp) {
-  register Tcl_CallFrame *varFramePtr = (Tcl_CallFrame *)Tcl_Interp_varFramePtr(interp);
+  register const Tcl_CallFrame *varFramePtr = (Tcl_CallFrame *)Tcl_Interp_varFramePtr(interp);
 
   nonnull_assert(interp != NULL);
 
@@ -749,19 +829,17 @@ CallStackFindActiveFilter(const Tcl_Interp *interp) {
  *----------------------------------------------------------------------
  * CallStackFindEnsembleCsc --
  *
- *    Return the callstack content and the optionally the stack frame
+ *    Return the call-stack content and the optionally the stack frame
  *    of the last ensemble invocation.
  *
  * Results:
- *    Callstack content
+ *    Call-stack content
  *
  * Side effects:
  *    None.
  *
  *----------------------------------------------------------------------
  */
-static NsfCallStackContent * CallStackFindEnsembleCsc(const Tcl_CallFrame *framePtr, Tcl_CallFrame **framePtrPtr) nonnull(1) nonnull(2);
-
 static NsfCallStackContent *
 CallStackFindEnsembleCsc(const Tcl_CallFrame *framePtr, Tcl_CallFrame **framePtrPtr) {
   register Tcl_CallFrame *varFramePtr;
@@ -778,19 +856,19 @@ CallStackFindEnsembleCsc(const Tcl_CallFrame *framePtr, Tcl_CallFrame **framePtr
     assert(cscPtr != NULL);
 
     /*fprintf(stderr,"	--- frame %p cmdPtr %p NSF_CSC_TYPE_ENSEMBLE %d NSF_CSC_CALL_IS_ENSEMBLE %d \
-			 NSF_CSC_TYPE_INACTIVE %d\n",
-	    varFramePtr,
-	    cscPtr->cmdPtr,
-	    (cscPtr->frameType & NSF_CSC_TYPE_ENSEMBLE) != 0,
-	    (cscPtr->flags & NSF_CSC_CALL_IS_ENSEMBLE) != 0,
-	    (cscPtr->frameType & NSF_CSC_TYPE_INACTIVE) != 0);*/
+                         NSF_CSC_TYPE_INACTIVE %d\n",
+            varFramePtr,
+            cscPtr->cmdPtr,
+            (cscPtr->frameType & NSF_CSC_TYPE_ENSEMBLE) != 0,
+            (cscPtr->flags & NSF_CSC_CALL_IS_ENSEMBLE) != 0,
+            (cscPtr->frameType & NSF_CSC_TYPE_INACTIVE) != 0);*/
     /*
-     * The "root" frame in a callstack branch resulting from an ensemble
+     * The "root" frame in a call-stack branch resulting from an ensemble
      * dispatch is not typed as an NSF_CSC_TYPE_ENSEMBLE frame, the call type
      * /is/ NSF_CSC_CALL_IS_ENSEMBLE.
      */
     if ((cscPtr->frameType & NSF_CSC_TYPE_ENSEMBLE) == 0u &&
-	(cscPtr->flags & NSF_CSC_CALL_IS_ENSEMBLE)) {
+        (cscPtr->flags & NSF_CSC_CALL_IS_ENSEMBLE)) {
       break;
     }
   }
@@ -800,6 +878,48 @@ CallStackFindEnsembleCsc(const Tcl_CallFrame *framePtr, Tcl_CallFrame **framePtr
   return cscPtr;
 }
 
+/*
+ *----------------------------------------------------------------------
+ * CallStackNextFrameOfType --
+ *
+ *    Return the next frame on the call stack being of a specified type. The
+ *    type is specified by a bitmask passed as flags.
+ *
+ * Results:
+ *    Tcl_CallFrame
+ *
+ * Side effects:
+ *    None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static Tcl_CallFrame *
+CallStackNextFrameOfType(Tcl_CallFrame *framePtr, unsigned int flags) {
+
+  nonnull_assert(framePtr != NULL);
+
+  do {
+    NsfCallStackContent *cscPtr = Tcl_CallFrame_clientData(framePtr);
+
+    if (cscPtr != NULL && (cscPtr->frameType & NSF_CSC_TYPE_ENSEMBLE) != 0u) {
+      (void)CallStackFindEnsembleCsc(framePtr, &framePtr);
+    }
+
+    if (((unsigned int)Tcl_CallFrame_isProcCallFrame(framePtr) & flags) != 0u) {
+      /*
+       * framePtr has already the return value.
+       */
+      break;
+    }
+
+    framePtr = Tcl_CallFrame_callerPtr(framePtr);
+
+  } while (framePtr != NULL);
+
+  return framePtr;
+}
+
 /*
  *----------------------------------------------------------------------
  * CallStackMethodPath --
@@ -815,9 +935,6 @@ CallStackFindEnsembleCsc(const Tcl_CallFrame *framePtr, Tcl_CallFrame **framePtr
  *
  *----------------------------------------------------------------------
  */
-static Tcl_Obj* CallStackMethodPath(Tcl_Interp *interp, Tcl_CallFrame *framePtr)
-  nonnull(1) nonnull(2);
-
 static Tcl_Obj*
 CallStackMethodPath(Tcl_Interp *interp, Tcl_CallFrame *framePtr) {
   int      elements;
@@ -837,13 +954,13 @@ CallStackMethodPath(Tcl_Interp *interp, Tcl_CallFrame *framePtr) {
     assert(cscPtr != NULL);
 
     /*fprintf(stderr,	"--- frame %p cmdPtr %p cmd %s NSF_CSC_TYPE_ENSEMBLE %d \
-			NSF_CSC_CALL_IS_ENSEMBLE %d NSF_CSC_TYPE_INACTIVE %d\n",
-	    framePtr,
-	    cscPtr->cmdPtr,
-	    Tcl_GetCommandName(interp, cscPtr->cmdPtr),
-	    (cscPtr->frameType & NSF_CSC_TYPE_ENSEMBLE) != 0,
-	    (cscPtr->flags & NSF_CSC_CALL_IS_ENSEMBLE) != 0,
-	    (cscPtr->frameType & NSF_CSC_TYPE_INACTIVE) != 0);*/
+                        NSF_CSC_CALL_IS_ENSEMBLE %d NSF_CSC_TYPE_INACTIVE %d\n",
+            framePtr,
+            cscPtr->cmdPtr,
+            Tcl_GetCommandName(interp, cscPtr->cmdPtr),
+            (cscPtr->frameType & NSF_CSC_TYPE_ENSEMBLE) != 0,
+            (cscPtr->flags & NSF_CSC_CALL_IS_ENSEMBLE) != 0,
+            (cscPtr->frameType & NSF_CSC_TYPE_INACTIVE) != 0);*/
 
     /*
      * The "ensemble" call type, we find applied to all intermediate and leaf
@@ -855,7 +972,7 @@ CallStackMethodPath(Tcl_Interp *interp, Tcl_CallFrame *framePtr) {
       break;
     }
     /*
-     * The callstack might contain consecutive calls of ensemble entry calls
+     * The call-stack might contain consecutive calls of ensemble entry calls
      * chained via next. We can detect consecutive calls via the elements
      * count.
      */
@@ -864,11 +981,11 @@ CallStackMethodPath(Tcl_Interp *interp, Tcl_CallFrame *framePtr) {
     }
 
     Tcl_ListObjAppendElement(interp, methodPathObj,
-			     Tcl_NewStringObj(Tcl_GetCommandName(interp, cscPtr->cmdPtr), -1));
+                             Tcl_NewStringObj(Tcl_GetCommandName(interp, cscPtr->cmdPtr), -1));
     elements++;
 
     /*
-     * The "root" frame in a callstack branch resulting from an ensemble
+     * The "root" frame in a call-stack branch resulting from an ensemble
      * dispatch is not typed as an NSF_CSC_TYPE_ENSEMBLE frame, the call type
      * /is/ NSF_CSC_CALL_IS_ENSEMBLE (as checked above).
      */
@@ -919,10 +1036,12 @@ CallStackMethodPath(Tcl_Interp *interp, Tcl_CallFrame *framePtr) {
  *
  *----------------------------------------------------------------------
  */
-NSF_INLINE static int FilterActiveOnObj(const Tcl_Interp *interp, const NsfObject *object, Tcl_Command cmd) nonnull(1) nonnull(2);
-
 NSF_INLINE static int
-FilterActiveOnObj(const Tcl_Interp *interp, const NsfObject *object, Tcl_Command cmd) {
+FilterActiveOnObj(
+    const Tcl_Interp *interp,
+    const NsfObject *object,
+    const Tcl_Command cmd
+) {
   register const Tcl_CallFrame *varFramePtr = (const Tcl_CallFrame *)Tcl_Interp_varFramePtr(interp);
 
   nonnull_assert(interp != NULL);
@@ -945,7 +1064,7 @@ FilterActiveOnObj(const Tcl_Interp *interp, const NsfObject *object, Tcl_Command
  * CallStackReplaceVarTableReferences --
  *
  *    Replace all references to the old var table (arg 1) by
- *    references to a new var table (arg 2) on the callstack.
+ *    references to a new var table (arg 2) on the call-stack.
  *    This function is e.g. used by require namespace.
  *
  * Results:
@@ -956,7 +1075,6 @@ FilterActiveOnObj(const Tcl_Interp *interp, const NsfObject *object, Tcl_Command
  *
  *----------------------------------------------------------------------
  */
-static void CallStackReplaceVarTableReferences(const Tcl_Interp *interp, TclVarHashTable *oldVarTablePtr, TclVarHashTable *newVarTablePtr) nonnull(1) nonnull(2) nonnull(3);
 
 static void
 CallStackReplaceVarTableReferences(const Tcl_Interp *interp, TclVarHashTable *oldVarTablePtr, TclVarHashTable *newVarTablePtr) {
@@ -988,9 +1106,9 @@ CallStackReplaceVarTableReferences(const Tcl_Interp *interp, TclVarHashTable *ol
  *----------------------------------------------------------------------
  * CallStackPopAll --
  *
- *    Unwind the stack and pop all callstack entries that are still
+ *    Unwind the stack and pop all call-stack entries that are still
  *    alive (e.g.  if "exit" is called and we were jumping out of the
- *    callframe).
+ *    call-frame).
  *
  * Results:
  *    None.
@@ -1074,16 +1192,8 @@ static void CallStackPopAll(Tcl_Interp *interp) {
  *
  *----------------------------------------------------------------------
  */
-#if defined(NRE)
-static NsfCallStackContent * CscAlloc(Tcl_Interp *interp, NsfCallStackContent *cscPtr, Tcl_Command cmd)
-  nonnull(1);
-#else
-static NsfCallStackContent * CscAlloc(Tcl_Interp *interp, NsfCallStackContent *cscPtr, Tcl_Command cmd)
-  nonnull(2);
-#endif
-
 static NsfCallStackContent *
-CscAlloc(Tcl_Interp *interp, NsfCallStackContent *cscPtr, Tcl_Command cmd) {
+CscAlloc(Tcl_Interp *interp, NsfCallStackContent *cscPtr, const Tcl_Command cmd) {
 #if defined(NRE)
   Tcl_ObjCmdProc *proc = (cmd != NULL) ? Tcl_Command_objProc(cmd) : NULL;
 
@@ -1119,12 +1229,15 @@ CscAlloc(Tcl_Interp *interp, NsfCallStackContent *cscPtr, Tcl_Command cmd) {
  *
  *----------------------------------------------------------------------
  */
-NSF_INLINE static void CscInit_(/*@notnull@*/ NsfCallStackContent *cscPtr, NsfObject *object, NsfClass *cl,
-	Tcl_Command cmd, unsigned short frameType, unsigned int flags) nonnull(1) nonnull(2);
-
 NSF_INLINE static void
-CscInit_(/*@notnull@*/ NsfCallStackContent *cscPtr, NsfObject *object, NsfClass *cl,
-	Tcl_Command cmd, unsigned short frameType, unsigned int flags) {
+CscInit_(
+    NsfCallStackContent *cscPtr,
+    NsfObject *object,
+    NsfClass *class,
+    const Tcl_Command cmd,
+    unsigned short frameType,
+    unsigned int flags
+) {
 #if defined(NSF_PROFILE)
   struct Tcl_Time trt;
 #endif
@@ -1151,17 +1264,17 @@ CscInit_(/*@notnull@*/ NsfCallStackContent *cscPtr, NsfObject *object, NsfClass
     object->activationCount ++;
     MEM_COUNT_ALLOC("object.activationCount",object);
     /*fprintf(stderr, "CscInit %p method %s activationCount ++ (%s) --> %d (cl %p)\n",
-	    cscPtr, (cmd != NULL) ? Tcl_GetCommandName(object->teardown, cmd) : "UNK",
-	    ObjectName(object),  object->activationCount, cl);*/
+            cscPtr, (cmd != NULL) ? Tcl_GetCommandName(object->teardown, cmd) : "UNK",
+            ObjectName(object),  object->activationCount, cl);*/
     /*
      * Track class activations
      */
-    if (cl != NULL) {
+    if (class != NULL) {
       /*
        * handle class activation count
        */
-      cl->object.activationCount ++;
-      MEM_COUNT_ALLOC("class.activationCount", cl);
+      class->object.activationCount ++;
+      MEM_COUNT_ALLOC("class.activationCount", class);
       /*
        * Increment the namespace ptr in case Tcl tries to delete
        * this namespace during the invocation
@@ -1174,7 +1287,7 @@ CscInit_(/*@notnull@*/ NsfCallStackContent *cscPtr, NsfObject *object, NsfClass
   }
   cscPtr->flags        |= flags & NSF_CSC_COPY_FLAGS;
   cscPtr->self          = object;
-  cscPtr->cl            = cl;
+  cscPtr->cl            = class;
   cscPtr->cmdPtr        = cmd;
   cscPtr->objv          = NULL;
   cscPtr->filterStackEntry = object->filterStack;
@@ -1199,8 +1312,6 @@ CscInit_(/*@notnull@*/ NsfCallStackContent *cscPtr, NsfObject *object, NsfClass
  *
  *----------------------------------------------------------------------
  */
-NSF_INLINE static void CscFinish_(Tcl_Interp *interp, NsfCallStackContent *cscPtr) nonnull(1) nonnull(2);
-
 NSF_INLINE static void
 CscFinish_(Tcl_Interp *interp, NsfCallStackContent *cscPtr) {
   NsfObject *object;
@@ -1247,10 +1358,10 @@ CscFinish_(Tcl_Interp *interp, NsfCallStackContent *cscPtr) {
     MEM_COUNT_FREE("object.activationCount", object);
 
     /*fprintf(stderr, "CscFinish decr activationCount for %s to %d object->flags %.6x dc %.6x succ %.6x\n",
-	    ObjectName(cscPtr->self), cscPtr->self->activationCount, object->flags,
-	    object->flags & NSF_DESTROY_CALLED,
-	    object->flags & NSF_DESTROY_CALLED_SUCCESS
-	    );*/
+            ObjectName(cscPtr->self), cscPtr->self->activationCount, object->flags,
+            object->flags & NSF_DESTROY_CALLED,
+            object->flags & NSF_DESTROY_CALLED_SUCCESS
+            );*/
 
     assert(object->activationCount > -1);
 
@@ -1269,14 +1380,14 @@ CscFinish_(Tcl_Interp *interp, NsfCallStackContent *cscPtr) {
       MEM_COUNT_FREE("class.activationCount", clObject);
 
       /*fprintf(stderr, "CscFinish class %p %s check ac %d flags destroy %.6x success %.6x\n",
-	      clObject, ObjectName(clObject),
-	      clObject->activationCount,
-	      clObject->flags & NSF_DESTROY_CALLED,
-	      clObject->flags & NSF_DESTROY_CALLED_SUCCESS);*/
+              clObject, ObjectName(clObject),
+              clObject->activationCount,
+              clObject->flags & NSF_DESTROY_CALLED,
+              clObject->flags & NSF_DESTROY_CALLED_SUCCESS);*/
 
       if (clObject->activationCount < 1 && clObject->flags & NSF_DESTROY_CALLED && allowDestroy) {
-	/* fprintf(stderr, "CscFinish calls destroy class %p\n", clObject);*/
-	CallStackDoDestroy(interp, clObject);
+        /* fprintf(stderr, "CscFinish calls destroy class %p\n", clObject);*/
+        CallStackDoDestroy(interp, clObject);
       }
       /*
        * Release the Namespace
@@ -1313,7 +1424,8 @@ CscFinish_(Tcl_Interp *interp, NsfCallStackContent *cscPtr) {
  *----------------------------------------------------------------------
  */
 #if 0
-static Tcl_CallFrame * BeginOfCallChain(const Tcl_Interp *interp, NsfObject *object) nonnull(1);
+static Tcl_CallFrame * BeginOfCallChain(const Tcl_Interp *interp, NsfObject *object)
+  nonnull(1);
 
 static Tcl_CallFrame *
 BeginOfCallChain(const Tcl_Interp *interp, NsfObject *object) {
@@ -1328,13 +1440,13 @@ BeginOfCallChain(const Tcl_Interp *interp, NsfObject *object) {
       register unsigned int flags = Tcl_CallFrame_isProcCallFrame(varFramePtr);
 
       if (flags & (FRAME_IS_NSF_METHOD|FRAME_IS_NSF_CMETHOD)) {
-	const NsfCallStackContent *cscPtr = (NsfCallStackContent *)Tcl_CallFrame_clientData(varFramePtr);
-	if (cscPtr->self == object) {
-	  prevFramePtr = varFramePtr;
-	  continue;
-	}
+        const NsfCallStackContent *cscPtr = (NsfCallStackContent *)Tcl_CallFrame_clientData(varFramePtr);
+        if (cscPtr->self == object) {
+          prevFramePtr = varFramePtr;
+          continue;
+        }
       } else if ((flags & (FRAME_IS_NSF_OBJECT|FRAME_IS_LAMBDA)) != 0u) {
-	continue;
+        continue;
       }
       break;
     }
