@@ -1,20 +1,17 @@
 /*
- * Experimental Interface to MongoDB based on nsf (Next Scripting
+ * Interface between MongoDB based on NSF (Next Scripting
  * Framework)
  *
- * The example shows how to use the source code generator from nsf to
- * generate a c interface.
-
- * This implementation provides a low level interface based on tagged
- * elements to force / preserve the datatypes of mongodb.
+ * This implementation provides a low level interface based on tagged elements
+ * to force / preserve the datatypes of MongoDB when converting into Tcl.
  *
- * This code serves as well as an example how to use the source code
- * generator of nsf to provide a C interface for optional nsf
- * packages.
+ * This code serves as well as an example how to use the source code generator
+ * of NSF.  The example shows how to use the source code generator from NSF to
+ * generate a C interface.
  *
  * -gustaf neumann    March 27, 2011
  *
- * Copyright (C) 2011-2016 Gustaf Neumann
+ * Copyright (C) 2011-2018 Gustaf Neumann
  */
 #include <stdlib.h>
 #include <stdio.h>
@@ -46,6 +43,7 @@ static mongoc_uri_t *mongoUri = NULL;
 
 typedef enum {
   NSF_BSON_ARRAY,
+  NSF_BSON_BINARY,
   NSF_BSON_BOOL,
   NSF_BSON_INT32,
   NSF_BSON_INT64,
@@ -63,9 +61,10 @@ typedef enum {
   NSF_BSON_UNKNOWN
 } nsfMongoTypes;
 
-static char *
+static const char *
 NsfMongoGlobalStrings[] = {
   "array",
+  "binary",
   "boolean",
   "int32",
   "int64",
@@ -85,6 +84,23 @@ NsfMongoGlobalStrings[] = {
 };
 static Tcl_Obj **NsfMongoGlobalObjs = NULL;
 
+static Tcl_Obj *BsonToList(Tcl_Interp *interp, const bson_t *data , int depth);
+static bson_type_t BsonTagToType(Tcl_Interp *interp, const char *tag);
+
+extern Tcl_PackageInitProc Nsfmongo_SafeInit;
+extern Tcl_PackageInitProc Nsfmongo_Init;
+static Tcl_ExitProc Nsfmongo_Exit;
+static Tcl_ExitProc Nsfmongo_ThreadExit;
+
+Nsf_TypeConverter Nsf_ConvertTo_Boolean;
+Nsf_TypeConverter Nsf_ConvertTo_Class;
+Nsf_TypeConverter Nsf_ConvertTo_Int32;
+Nsf_TypeConverter Nsf_ConvertTo_Integer;
+Nsf_TypeConverter Nsf_ConvertTo_Object;
+Nsf_TypeConverter Nsf_ConvertTo_Pointer;
+Nsf_TypeConverter Nsf_ConvertTo_String;
+Nsf_TypeConverter Nsf_ConvertTo_Tclobj;
+
 /***********************************************************************
  * The following definitions should not be here, but they are included
  * to get compilation going for the time being.
@@ -93,18 +109,18 @@ typedef void *NsfObject;
 
 #define PARSE_CONTEXT_PREALLOC 20
 typedef struct {
-  int status;
-  ClientData *clientData; /* 4 members pointer to the actual parse context data */
-  Tcl_Obj **objv;
-  Tcl_Obj **full_objv;    /* contains method as well */
-  int *flags;
-  ClientData clientData_static[PARSE_CONTEXT_PREALLOC]; /* 3 members preallocated parse context data */
-  Tcl_Obj *objv_static[PARSE_CONTEXT_PREALLOC+1];
-  int flags_static[PARSE_CONTEXT_PREALLOC+1];
-  int lastObjc;           /* points to the first "unprocessed" argument */
-  int objc;
-  int varArgs;            /* does the parameter end with some kind of "args" */
-  NsfObject *object;
+  ClientData   *clientData;   /* 4 members pointer to the actual parse context data */
+  Tcl_Obj     **objv;
+  Tcl_Obj     **full_objv;    /* contains method as well */
+  unsigned int *flags;
+  ClientData    clientData_static[PARSE_CONTEXT_PREALLOC]; /* 3 members preallocated parse context data */
+  Tcl_Obj      *objv_static[PARSE_CONTEXT_PREALLOC+1];
+  unsigned int  flags_static[PARSE_CONTEXT_PREALLOC+1];
+  unsigned int  status;
+  int           lastObjc;     /* points to the first "unprocessed" argument */
+  int           objc;
+  NsfObject    *object;
+  int           varArgs;      /* does the parameter end with some kind of "args" */
 } ParseContext;
 
 #define nr_elements(arr)  ((int) (sizeof(arr) / sizeof(arr[0])))
@@ -144,9 +160,9 @@ typedef struct {
 #endif
 
 
-static int ArgumentParse(Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[],
+static int ArgumentParse(Tcl_Interp *interp, int objc, Tcl_Obj *const objv[],
                          NsfObject *obj, Tcl_Obj *procName,
-                         Nsf_Param CONST *paramPtr, int nrParameters, int serial,
+                         Nsf_Param const *paramPtr, int nrParameters, int serial,
                          unsigned int processFlags, ParseContext *pc) {
   return Nsf_ArgumentParse(interp, objc, objv, (Nsf_Object *)obj,
                            procName, paramPtr, nrParameters, serial,
@@ -154,7 +170,7 @@ static int ArgumentParse(Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[],
 }
 
 /***********************************************************************
- * Include the generated mongo db api.
+ * Include the generated mongo db API.
  ***********************************************************************/
 
 #include "mongoAPI.h"
@@ -206,7 +222,7 @@ BsonToList(Tcl_Interp *interp, const bson_t *data , int depth)
     case BSON_TYPE_DOUBLE:    tag = NSF_BSON_DOUBLE;    elemObj = Tcl_NewDoubleObj(bson_iter_double( &i )); break;
     case BSON_TYPE_BOOL:      tag = NSF_BSON_BOOL;      elemObj = Tcl_NewBooleanObj(bson_iter_bool( &i )); break;
     case BSON_TYPE_REGEX:  {
-      const char *options = NULL, *regex = NULL;
+      const char *options = NULL, *regex;
 
       tag = NSF_BSON_REGEX;
       regex = bson_iter_regex( &i, &options );
@@ -248,7 +264,7 @@ BsonToList(Tcl_Interp *interp, const bson_t *data , int depth)
       bson_t         b;
 
       tag = NSF_BSON_DOCUMENT;
-      bson_iter_document (&i, &doclen, &docbuf);
+      bson_iter_document(&i, &doclen, &docbuf);
       bson_init_static(&b, docbuf, doclen);
       elemObj = BsonToList(interp, &b , depth + 1 );
       break;
@@ -260,7 +276,7 @@ BsonToList(Tcl_Interp *interp, const bson_t *data , int depth)
 
       tag = NSF_BSON_ARRAY;
       bson_iter_array(&i, &doclen, &docbuf);
-      bson_init_static (&b, docbuf, doclen);
+      bson_init_static(&b, docbuf, doclen);
       elemObj = BsonToList(interp, &b , depth + 1 );
       break;
     }
@@ -269,12 +285,27 @@ BsonToList(Tcl_Interp *interp, const bson_t *data , int depth)
       char              string[BSON_DECIMAL128_STRING];
 
       tag = NSF_BSON_DECIMAL128;
-      bson_iter_decimal128( &i,  &decimal128);
-      bson_decimal128_to_string (&decimal128, string);
+      bson_iter_decimal128( &i, &decimal128);
+      bson_decimal128_to_string(&decimal128, string);
       elemObj = Tcl_NewStringObj(string, -1);
 
       break;
     }
+    case BSON_TYPE_BINARY: {
+      uint32_t       length;
+      const uint8_t *bytes;
+
+      tag = NSF_BSON_BINARY;
+      bson_iter_binary( &i, NULL /* subtype_t */, &length, &bytes);
+      elemObj = Tcl_NewByteArrayObj(bytes, (int)length);
+      break;
+    }
+    case BSON_TYPE_CODE:       /* fall through */
+    case BSON_TYPE_CODEWSCOPE: /* fall through */
+    case BSON_TYPE_DBPOINTER:  /* fall through */
+    case BSON_TYPE_EOD:        /* fall through */
+    case BSON_TYPE_SYMBOL:     /* fall through */
+    case BSON_TYPE_UNDEFINED:  /* fall through */
     default:
       tag = NSF_BSON_UNKNOWN;
       elemObj = Tcl_NewStringObj("", 0);
@@ -308,7 +339,7 @@ BsonToList(Tcl_Interp *interp, const bson_t *data , int depth)
  *----------------------------------------------------------------------
  */
 bson_type_t
-BsonTagToType(Tcl_Interp *interp, CONST char *tag)
+BsonTagToType(Tcl_Interp *interp, const char *tag)
 {
   char firstChar = *tag;
 
@@ -358,7 +389,7 @@ BsonTagToType(Tcl_Interp *interp, CONST char *tag)
  *----------------------------------------------------------------------
  */
 static int
-BsonAppend(Tcl_Interp *interp, bson_t *bbPtr, CONST char *name, CONST char *tag, Tcl_Obj *value)
+BsonAppend(Tcl_Interp *interp, bson_t *bbPtr, const char *name, const char *tag, Tcl_Obj *value)
 {
   int         result = TCL_OK;
   bson_type_t t = BsonTagToType(interp, tag);
@@ -444,19 +475,21 @@ BsonAppend(Tcl_Interp *interp, bson_t *bbPtr, CONST char *name, CONST char *tag,
     break;
   }
   case BSON_TYPE_TIMESTAMP: {
-    int       timestamp, increment, objc = 0;
+    int       timestamp = 0, increment = 0, objc = 0;
     Tcl_Obj **objv;
 
     result = Tcl_ListObjGetElements(interp, value, &objc, &objv);
     if (result != TCL_OK || objc != 2) {
       return NsfPrintError(interp, "invalid timestamp: %s", ObjStr(value));
+    } else {
+      result = Tcl_GetIntFromObj(interp, objv[0], &timestamp);
+      if (result == TCL_OK) {
+        result = Tcl_GetIntFromObj(interp, objv[1], &increment);
+      }
+      if (result == TCL_OK) {
+        bson_append_timestamp(bbPtr, name, keyLength, (uint32_t)timestamp, (uint32_t)increment);
+      }
     }
-    result = Tcl_GetIntFromObj(interp, objv[0], &timestamp);
-    if (result == TCL_OK) {
-      result = Tcl_GetIntFromObj(interp, objv[1], &increment);
-    }
-    if (result != TCL_OK) break;
-    bson_append_timestamp(bbPtr, name, keyLength, (uint32_t)timestamp, (uint32_t)increment);
     break;
   }
   case BSON_TYPE_DOCUMENT:
@@ -492,11 +525,17 @@ BsonAppend(Tcl_Interp *interp, bson_t *bbPtr, CONST char *name, CONST char *tag,
   case BSON_TYPE_DECIMAL128: {
     bson_decimal128_t decimal128;
 
-    bson_decimal128_from_string (ObjStr(value), &decimal128);
+    bson_decimal128_from_string(ObjStr(value), &decimal128);
     bson_append_decimal128(bbPtr, name, keyLength, &decimal128);
     break;
   }
-  case BSON_TYPE_BINARY:
+  case BSON_TYPE_BINARY: {
+    int            length;
+    const uint8_t *data = Tcl_GetByteArrayFromObj(value, &length);
+    bson_append_binary(bbPtr, name, keyLength, 0x00 /*bson_subtype_t*/,
+                       data, (uint32_t)length);
+    break;
+  }
   case BSON_TYPE_DBPOINTER:
   case BSON_TYPE_CODE:
   case BSON_TYPE_SYMBOL:
@@ -550,7 +589,7 @@ BsonAppendObjv(Tcl_Interp *interp, bson_t *bPtr, int objc, Tcl_Obj **objv)
 
 
 /***********************************************************************
- * Define the api functions
+ * Define the API functions
  ***********************************************************************/
 /*
   cmd json::generate NsfMongoJsonGenerate {
@@ -562,7 +601,6 @@ NsfMongoJsonGenerate(Tcl_Interp *interp, Tcl_Obj *listObj)
 {
   bson_t    list, *listPtr = &list;
   size_t    length;
-  char     *jsonString;
   int       result, objc;
   Tcl_Obj **objv;
 
@@ -573,6 +611,8 @@ NsfMongoJsonGenerate(Tcl_Interp *interp, Tcl_Obj *listObj)
 
   result = BsonAppendObjv(interp, listPtr, objc, objv);
   if (result == TCL_OK) {
+    char     *jsonString;
+
     jsonString = bson_as_json(listPtr, &length);
     if (jsonString != NULL) {
       Tcl_SetObjResult(interp, Tcl_NewStringObj(jsonString, (int)length));
@@ -618,25 +658,25 @@ NsfMongoJsonParse(Tcl_Interp *interp, Tcl_Obj *jsonObj)
   }
 */
 static int
-NsfMongoClose(Tcl_Interp *interp, mongoc_client_t *clientPtr, Tcl_Obj *clientObj)
+NsfMongoClose(Tcl_Interp *UNUSED(interp), mongoc_client_t *connPtr, Tcl_Obj *connObj)
 {
 #if defined(USE_CLIENT_POOL)
-  mongoc_client_pool_push(mongoClientPool, clientPtr);
+  mongoc_client_pool_push(mongoClientPool, connPtr);
 #else
-  mongoc_client_destroy(clientPtr);
+  mongoc_client_destroy(connPtr);
 #endif
-  Nsf_PointerDelete(ObjStr(clientObj), clientPtr, 0);
+  Nsf_PointerDelete(ObjStr(connObj), connPtr, 0);
 
   return TCL_OK;
 }
 
 /*
   cmd connect NsfMongoConnect {
-  {-argName "-uri" -required 1 -nrargs 1}
+  {-argName "-uri" -required 0 -nrargs 1}
   }
 */
 static int
-NsfMongoConnect(Tcl_Interp *interp, CONST char *uri)
+NsfMongoConnect(Tcl_Interp *interp, const char *uri)
 {
   char             channelName[80];
   mongoc_client_t *clientPtr;
@@ -668,7 +708,7 @@ NsfMongoConnect(Tcl_Interp *interp, CONST char *uri)
    * Make an entry in the symbol table and return entry name it as
    * result.
    */
-  if (Nsf_PointerAdd(interp, channelName, "mongoc_client_t", clientPtr) != TCL_OK) {
+  if (Nsf_PointerAdd(interp, channelName, 80u, "mongoc_client_t", clientPtr) != TCL_OK) {
     mongoc_client_destroy(clientPtr);
     return TCL_ERROR;
   }
@@ -688,7 +728,7 @@ NsfMongoConnect(Tcl_Interp *interp, CONST char *uri)
 */
 static int
 NsfMongoRunCmd(Tcl_Interp *interp, int withNocomplain, mongoc_client_t *clientPtr,
-               CONST char *db, Tcl_Obj *cmdObj)
+               const char *db, Tcl_Obj *cmdObj)
 {
   bson_t               cmd, *cmdPtr = &cmd, reply, *replyPtr = &reply;
   mongoc_read_prefs_t *readPrefsPtr = NULL; /* TODO: not used */
@@ -723,14 +763,20 @@ NsfMongoRunCmd(Tcl_Interp *interp, int withNocomplain, mongoc_client_t *clientPt
   }
 */
 static int
-NsfMongoStatus(Tcl_Interp *interp, mongoc_client_t *clientPtr, Tcl_Obj *clientObj)
+NsfMongoStatus(Tcl_Interp *interp, mongoc_client_t *clientPtr, Tcl_Obj *UNUSED(clientObj))
 {
   mongoc_read_prefs_t *readPrefs = NULL; /* TODO: not handled */
   bson_t               reply, *replyPtr = &reply;
   bson_error_t         bsonError;
   int                  result = TCL_OK;
+  bson_t               cmd = BSON_INITIALIZER;
+  bool                 ret;
 
-  if (likely(mongoc_client_get_server_status(clientPtr, readPrefs, replyPtr, &bsonError)) !=0) {
+  BSON_APPEND_INT32(&cmd, "serverStatus", 1);
+  ret = mongoc_client_command_simple(clientPtr, "admin", &cmd, readPrefs, replyPtr, &bsonError);
+  bson_destroy(&cmd);
+
+  if (likely(ret != 0)) {
     Tcl_SetObjResult(interp, BsonToList(interp, replyPtr, 0));
   } else {
     result = NsfPrintError(interp, "mongo::status: error: %s", bsonError.message);
@@ -761,7 +807,7 @@ NsfCollectionOpen(Tcl_Interp *interp,
   if (collectionPtr != NULL) {
     char buffer[80];
 
-    if (Nsf_PointerAdd(interp, buffer, "mongoc_collection_t", collectionPtr) == TCL_OK) {
+    if (Nsf_PointerAdd(interp, buffer, 80u, "mongoc_collection_t", collectionPtr) == TCL_OK) {
       Tcl_SetObjResult(interp, Tcl_NewStringObj(buffer, -1));
       result = TCL_OK;
     } else {
@@ -785,7 +831,7 @@ NsfCollectionOpen(Tcl_Interp *interp,
   }
 */
 static int
-NsfCollectionClose(Tcl_Interp *interp, mongoc_collection_t *collectionPtr, Tcl_Obj *clientObj)
+NsfCollectionClose(Tcl_Interp *UNUSED(interp), mongoc_collection_t *collectionPtr, Tcl_Obj *clientObj)
 {
   mongoc_collection_destroy(collectionPtr);
   Nsf_PointerDelete(ObjStr(clientObj), collectionPtr, 0);
@@ -809,6 +855,7 @@ NsfMongoCollectionCount(Tcl_Interp *interp,
   Tcl_Obj    **objv;
   bson_t       query, *queryPtr = &query;
   bson_error_t bsonError;
+  /*bson_t* opts = BCON_NEW("skip", BCON_INT64(5));*/
 
   result = Tcl_ListObjGetElements(interp, queryObj, &objc, &objv);
   if (result != TCL_OK || ((objc % 3) != 0)) {
@@ -817,11 +864,12 @@ NsfMongoCollectionCount(Tcl_Interp *interp,
 
   BsonAppendObjv(interp, queryPtr, objc, objv);
 
-  count = mongoc_collection_count(collectionPtr,
-                                  0 /* query flags */, queryPtr,
-                                  0 /*skip */, 0 /*limit */,
-                                  NULL /* read preferences */,
-                                  &bsonError);
+  count = mongoc_collection_count_documents(collectionPtr,
+                                            queryPtr,
+                                            NULL /* opts */,
+                                            NULL /* read preferences */,
+                                            NULL /* replyPtr */,
+                                            &bsonError);
   if (count == -1) {
     bson_destroy( queryPtr );
     return NsfPrintError(interp, "mongo::collection::count: error: %s", bsonError.message);
@@ -884,7 +932,7 @@ static int
 NsfMongoCollectionIndex(Tcl_Interp *interp,
                         mongoc_collection_t *collectionPtr,
                         Tcl_Obj *attributesObj,
-                        CONST char *withName,
+                        const char *withName,
                         int withBackground,
                         int withDropdups,
                         int withSparse,
@@ -896,6 +944,9 @@ NsfMongoCollectionIndex(Tcl_Interp *interp,
   bson_t             keys, *keysPtr = &keys;
   bson_error_t       bsonError;
   mongoc_index_opt_t options;
+  bson_t            *create_indexes;
+  char              *index_name;
+  const char        *collection_name;
 
   result = Tcl_ListObjGetElements(interp, attributesObj, &objc, &objv);
   if (result != TCL_OK || ((objc % 3) != 0)) {
@@ -904,6 +955,20 @@ NsfMongoCollectionIndex(Tcl_Interp *interp,
 
   BsonAppendObjv(interp, keysPtr, objc, objv);
 
+  index_name = mongoc_collection_keys_to_index_string(keysPtr);
+  collection_name = mongoc_collection_get_name(collectionPtr);
+
+  create_indexes = BCON_NEW("createIndexes",
+                            BCON_UTF8(collection_name),
+                            "indexes",
+                            "[",
+                            "{",
+                            "key",
+                            BCON_DOCUMENT(keysPtr),
+                            "name",
+                            BCON_UTF8(index_name),
+                            "}",
+                            "]");
   mongoc_index_opt_init(&options);
 
   if (withBackground != 0) {options.background = 1;}
@@ -912,11 +977,17 @@ NsfMongoCollectionIndex(Tcl_Interp *interp,
   if (withUnique != 0)     {options.unique = 1;}
   if (withTtl != 0)        {options.expire_after_seconds = withTtl;}
   if (withName != 0)       {options.name = withName;}
-  /* TODO: not handled: is_initialized, v, weights, default_language, laguage_override, padding */
-
-  success = mongoc_collection_create_index(collectionPtr, keysPtr, &options, &bsonError);
+  /* TODO: not handled: is_initialized, v, weights, default_language, language_override, padding */
 
+  success = mongoc_collection_write_command_with_opts(
+                                                    collectionPtr,
+                                                    create_indexes,
+                                                    NULL /* opts */,
+                                                    NULL /*&reply*/,
+                                                    &bsonError);
   bson_destroy(keysPtr);
+  bson_free(index_name);
+  bson_destroy(create_indexes);
 
   Tcl_SetObjResult(interp, Tcl_NewBooleanObj(success));
   return TCL_OK;
@@ -957,8 +1028,8 @@ static int NsfMongoCollectionInsert(Tcl_Interp *interp,
   bson_append_oid(bsonPtr, "_id", 3, &oid);
 
   for (i = 0; i < objc; i += 3) {
-    CONST char *name  = ObjStr(objv[i]);
-    CONST char *tag   = ObjStr(objv[i+1]);
+    const char *name  = ObjStr(objv[i]);
+    const char *tag   = ObjStr(objv[i+1]);
     Tcl_Obj    *value = objv[i+2];
 
     /*fprintf(stderr, "adding pair '%s' (%s) '%s'\n", name, tag, ObjStr(value));*/
@@ -1048,14 +1119,16 @@ NsfMongoCollectionStats(Tcl_Interp *interp,
                         mongoc_collection_t *collectionPtr,
                         Tcl_Obj *optionsObj)
 {
-  int          objc = 0, success;
+  int          objc = 0, success, result;
   Tcl_Obj    **objv = NULL;
   bson_t       options, *optionsPtr = NULL;
   bson_t       stats, *statsPtr = &stats;
+  bson_t       cmd = BSON_INITIALIZER;
+  bson_iter_t  iter;
   bson_error_t bsonError;
 
   if (optionsObj != NULL) {
-    int result = Tcl_ListObjGetElements(interp, optionsObj, &objc, &objv);
+    result = Tcl_ListObjGetElements(interp, optionsObj, &objc, &objv);
 
     if (result != TCL_OK || ((objc % 3) != 0)) {
       return NsfPrintError(interp, "%s: must contain a multiple of 3 elements", ObjStr(optionsObj));
@@ -1064,19 +1137,40 @@ NsfMongoCollectionStats(Tcl_Interp *interp,
     BsonAppendObjv(interp, optionsPtr, objc, objv);
   }
 
-  success = mongoc_collection_stats(collectionPtr, optionsPtr, statsPtr, &bsonError);
+  if (optionsPtr != NULL && bson_iter_init_find(&iter, optionsPtr, "scale")
+      && !BSON_ITER_HOLDS_INT32 (&iter)) {
+    bson_set_error(&bsonError,
+                    MONGOC_ERROR_BSON,
+                    MONGOC_ERROR_BSON_INVALID,
+                    "'scale' must be an int32 value.");
+    success = 0;
+  } else {
+
+    BSON_APPEND_UTF8(&cmd, "collStats", mongoc_collection_get_name(collectionPtr));
+
+    if (optionsPtr != NULL) {
+      bson_concat(&cmd, optionsPtr);
+    }
+    success = mongoc_collection_command_simple(collectionPtr,
+                                               &cmd,
+                                               mongoc_collection_get_read_prefs(collectionPtr),
+                                               statsPtr,
+                                               &bsonError);
+    bson_destroy(&cmd);
+  }
 
   if (optionsPtr != NULL) {
-    bson_destroy (optionsPtr);
+    bson_destroy(optionsPtr);
   }
 
   if (success != 0) {
     Tcl_SetObjResult(interp, BsonToList(interp, statsPtr, 0));
-    bson_destroy (statsPtr);
-    return TCL_OK;
+    bson_destroy(statsPtr);
+    result = TCL_OK;
   } else {
-    return NsfPrintError(interp, "mongo::collection::stats: error: %s", bsonError.message);
+    result = NsfPrintError(interp, "mongo::collection::stats: error: %s", bsonError.message);
   }
+  return result;
 }
 
 /*
@@ -1191,7 +1285,7 @@ NsfMongoCursorAggregate(Tcl_Interp *interp,
   if (cursor != NULL) {
     char buffer[80];
 
-    if (Nsf_PointerAdd(interp, buffer, "mongoc_cursor_t", cursor) == TCL_OK) {
+    if (Nsf_PointerAdd(interp, buffer, 80u, "mongoc_cursor_t", cursor) == TCL_OK) {
       Tcl_SetObjResult(interp, Tcl_NewStringObj(buffer, -1));
     } else {
       mongoc_cursor_destroy( cursor );
@@ -1251,7 +1345,7 @@ NsfMongoCursorFind(Tcl_Interp *interp,
 
   if (cursor != NULL) {
     char buffer[80];
-    if (Nsf_PointerAdd(interp, buffer, "mongoc_cursor_t", cursor) == TCL_OK) {
+    if (Nsf_PointerAdd(interp, buffer, 80u, "mongoc_cursor_t", cursor) == TCL_OK) {
       Tcl_SetObjResult(interp, Tcl_NewStringObj(buffer, -1));
     } else {
       mongoc_cursor_destroy( cursor );
@@ -1292,7 +1386,7 @@ NsfMongoCursorNext(Tcl_Interp *interp, mongoc_cursor_t *cursor)
   }
 */
 static int
-NsfMongoCursorClose(Tcl_Interp *interp, mongoc_cursor_t *cursor, Tcl_Obj *cursorObj)
+NsfMongoCursorClose(Tcl_Interp *UNUSED(interp), mongoc_cursor_t *cursor, Tcl_Obj *cursorObj)
 {
   mongoc_cursor_destroy( cursor );
   Nsf_PointerDelete(ObjStr(cursorObj), cursor, 0);
@@ -1312,7 +1406,7 @@ NsfMongoCursorClose(Tcl_Interp *interp, mongoc_cursor_t *cursor, Tcl_Obj *cursor
   }
 */
 static int
-NsfMongoGridFSClose(Tcl_Interp *interp, mongoc_gridfs_t *gridfsPtr, Tcl_Obj *gridfsObj)
+NsfMongoGridFSClose(Tcl_Interp *UNUSED(interp), mongoc_gridfs_t *gridfsPtr, Tcl_Obj *gridfsObj)
 {
   mongoc_gridfs_destroy(gridfsPtr);
   Nsf_PointerDelete(ObjStr(gridfsObj), gridfsPtr, 0);
@@ -1330,7 +1424,7 @@ NsfMongoGridFSClose(Tcl_Interp *interp, mongoc_gridfs_t *gridfsPtr, Tcl_Obj *gri
 
 static int
 NsfMongoGridFSOpen(Tcl_Interp *interp, mongoc_client_t *clientPtr,
-                   CONST char *dbname, CONST char *prefix)
+                   const char *dbname, const char *prefix)
 {
   char             buffer[80];
   int              result = TCL_OK;
@@ -1343,7 +1437,7 @@ NsfMongoGridFSOpen(Tcl_Interp *interp, mongoc_client_t *clientPtr,
     result = NsfPrintError(interp, "mongo::gridfs::open: error: %s", bsonError.message);
   }
 
-  if (Nsf_PointerAdd(interp, buffer, "mongoc_gridfs_t", gfsPtr) == TCL_OK) {
+  if (Nsf_PointerAdd(interp, buffer, 80u, "mongoc_gridfs_t", gfsPtr) == TCL_OK) {
     Tcl_SetObjResult(interp, Tcl_NewStringObj(buffer, -1));
   } else {
     mongoc_gridfs_destroy(gfsPtr);
@@ -1374,16 +1468,18 @@ static int
 NsfMongoGridFileCreate(Tcl_Interp *interp,
                        GridfilesourceIdx_t withSource,
                        mongoc_gridfs_t *gridfsPtr,
-                       CONST char *value, CONST char *name,
-                       CONST char *contenttype,
+                       const char *value, const char *name,
+                       const char *contenttype,
                        Tcl_Obj *withMetadata
                        )
 {
   int                      result = TCL_OK;
-  mongoc_gridfs_file_opt_t fileOpts = {NULL};
+  mongoc_gridfs_file_opt_t fileOpts ;
   mongoc_gridfs_file_t    *gridFile;
   bson_t                   bsonMetaData, *bsonMetaDataPtr = &bsonMetaData;
 
+  memset(&fileOpts, 0, sizeof(fileOpts));
+
   if (withSource == GridfilesourceNULL) {
     withSource = GridfilesourceFileIdx;
   }
@@ -1411,7 +1507,7 @@ NsfMongoGridFileCreate(Tcl_Interp *interp,
 
   if (withSource == GridfilesourceFileIdx) {
     uint8_t buf[MONGOC_GRIDFS_READ_CHUNK];
-    struct iovec iov = { buf, 0 };
+    mongoc_iovec_t iov = { buf, 0 };
     int fd = open(value, O_RDONLY);
 
     if (fd < 1) {
@@ -1437,7 +1533,7 @@ NsfMongoGridFileCreate(Tcl_Interp *interp,
     }
     close(fd);
   } else {
-    struct iovec iov = { (char *)value, strlen(value) };
+    mongoc_iovec_t iov = { (char *)value, strlen(value) };
     mongoc_gridfs_file_writev(gridFile, &iov, 1, 0);
   }
   if (result == TCL_OK) {
@@ -1552,7 +1648,7 @@ NsfMongoGridFileOpen(Tcl_Interp *interp,
   if (gridFilePtr != NULL) {
     char buffer[80];
 
-    if (Nsf_PointerAdd(interp, buffer, "mongoc_gridfs_file_t", gridFilePtr) == TCL_OK) {
+    if (Nsf_PointerAdd(interp, buffer, 80u, "mongoc_gridfs_file_t", gridFilePtr) == TCL_OK) {
       Tcl_SetObjResult(interp, Tcl_NewStringObj(buffer, -1));
     } else {
       mongoc_gridfs_file_destroy(gridFilePtr);
@@ -1580,7 +1676,7 @@ NsfMongoGridFileOpen(Tcl_Interp *interp,
   }
 */
 static int
-NsfMongoGridFileClose(Tcl_Interp *interp, mongoc_gridfs_file_t* gridFilePtr, Tcl_Obj *gridFileObj)
+NsfMongoGridFileClose(Tcl_Interp *UNUSED(interp), mongoc_gridfs_file_t* gridFilePtr, Tcl_Obj *gridFileObj)
 {
   mongoc_gridfs_file_destroy(gridFilePtr);
   Nsf_PointerDelete(ObjStr(gridFileObj), gridFilePtr, 0);
@@ -1646,7 +1742,7 @@ NsfMongoGridFileRead(Tcl_Interp *interp, mongoc_gridfs_file_t *gridFilePtr, int
 {
   ssize_t      readSize;
   Tcl_Obj     *resultObj = Tcl_NewByteArrayObj(NULL, size);
-  struct iovec iov = { NULL, (size_t)size };
+  mongoc_iovec_t iov = { NULL, (size_t)size };
 
   assert(size > 0);
 
@@ -1669,11 +1765,13 @@ NsfMongoGridFileRead(Tcl_Interp *interp, mongoc_gridfs_file_t *gridFilePtr, int
   }
 */
 static int
-NsfMongoGridFileSeek(Tcl_Interp *interp, mongoc_gridfs_file_t *gridFilePtr, int offset)
+NsfMongoGridFileSeek(Tcl_Interp *UNUSED(interp), mongoc_gridfs_file_t *gridFilePtr, int offset)
 {
   int result;
 
-  /* TODO: whence SEEK_SET, SEEK_CUR or SEEK_END; implementation of SEEK_END looks incorrect */
+  /*
+   * TODO: whence SEEK_SET, SEEK_CUR or SEEK_END; implementation of SEEK_END looks incorrect
+   */
   result = mongoc_gridfs_file_seek(gridFilePtr, offset, SEEK_SET);
 
   return result < 0 ? TCL_ERROR : TCL_OK;
@@ -1683,11 +1781,11 @@ NsfMongoGridFileSeek(Tcl_Interp *interp, mongoc_gridfs_file_t *gridFilePtr, int
  * Finally, provide the necessary Tcl package interface.
  ***********************************************************************/
 
-void
-Nsfmongo_ThreadExit(ClientData clientData)
+static void
+Nsfmongo_ThreadExit(ClientData UNUSED(clientData))
 {
   /*
-   * The exit might happen at a time, when tcl is already shut down.
+   * The exit might happen at a time, when Tcl is already shut down.
    * We can't reliably call NsfLog.
    */
 
@@ -1709,11 +1807,11 @@ Nsfmongo_ThreadExit(ClientData clientData)
 #endif
 }
 
-void
+static void
 Nsfmongo_Exit(ClientData clientData)
 {
   /*
-   * The exit might happen at a time, when tcl is already shut down.
+   * The exit might happen at a time, when Tcl is already shut down.
    * We can't reliably call NsfLog.
    *
    *   Tcl_Interp *interp = (Tcl_Interp *)clientData;
@@ -1734,7 +1832,6 @@ Nsfmongo_Exit(ClientData clientData)
 }
 
 
-
 extern int
 Nsfmongo_Init(Tcl_Interp * interp)
 {
@@ -1825,9 +1922,7 @@ Nsfmongo_Init(Tcl_Interp * interp)
 }
 
 extern int
-Nsfmongo_SafeInit(interp)
-     Tcl_Interp *interp;
-{
+Nsfmongo_SafeInit( Tcl_Interp *interp) {
   return Nsfmongo_Init(interp);
 }
 
