@@ -1,22 +1,19 @@
 #
-# Object orientend mapping between MongoDB and nx. 
+# Object orientend mapping between MongoDB and NX.
 #
 # Gustaf Neumann              fecit, April 2011
 #
 package require nx
 package require nsf::mongo
-package provide nx::mongo 2.1
+package provide nx::mongo 2.2
 
-# todo: how to handle multiple connections; currently we have a single, global connection
-# todo: all references are currently auto-fetched. make this optional
-# todo: If "embeds" or "references" are used, the object must be of
-#       the specified classes, no subclasses allowed
-# todo: extend the query language syntax, e.g. regexp, ...
-# todo: handle remove for non-multivalued embedded objects
-# idea: handle names of nx objects (e.g. property like __name)
-# idea: handle classes von nx objects (e.g. property like __class)
-# idea: combine incremental slot operations with e.g. add -> $push, remove -> $pull
-# todo: make "embedded", "reference" spec even nicer?
+# todo: How to handle multiple connections; currently we have a single, global connection,
+# todo: All references are currently auto-fetched. Make this optional.
+# todo: Extend the query language syntax, e.g. regexp, ... (handled at least partly via "~" operator)
+# todo: Handle remove for non-multivalued embedded objects
+# idea: Handle names of nx objects (e.g. property like __name)
+# idea: Combine incremental slot operations with e.g. add -> $push, remove -> $pull
+# todo: Make "embedded", "reference" spec even nicer?
 
 namespace eval ::nx::mongo {
 
@@ -86,7 +83,7 @@ namespace eval ::nx::mongo {
 
     #
     # GridFS
-    # 
+    #
     :object property gridFs
 
     :public object method "gridfs open" {{name fs}} {
@@ -136,7 +133,7 @@ namespace eval ::nx::mongo {
       mongo::gridfile::close $f
       return $content
     }
-    
+
     :public object method "gridfs set attribute" {query attribute value} {
       set info [::nx::mongo::db gridfs list $query]
       if {$info eq ""} {return -code error "no such file <$query> stored in gridfs"}
@@ -147,7 +144,7 @@ namespace eval ::nx::mongo {
       } elseif {[dict exists $d $attribute]} {
         # wrong value replace it
         set bson {}
-        foreach {att type v} $info { 
+        foreach {att type v} $info {
           if {$att eq $attribute} {
             lappend bson $att $type $value
           } else {
@@ -187,20 +184,20 @@ namespace eval ::nx::mongo {
 
 
   }
-  
+
   #######################################################################
   # nx::mongo::Attribute is a specialized property slot
   #
   ::nx::MetaSlot create ::nx::mongo::Attribute -superclass ::nx::VariableSlot {
     :property mongotype
     :property rep
-    
+
     #
     # manage logging of mongo concerns
     #
     :public method log {msg} {
       if {$::nx::mongo::log} {
-	nsf::log notice "mongo-attribute: $msg"
+        nsf::log notice "mongo-attribute: $msg"
       }
     }
 
@@ -210,52 +207,52 @@ namespace eval ::nx::mongo {
       # from "type". Not all types are mappable easily to mongo types.
       #
       if {![info exists :mongotype]} {
-	set :mongotype string
-	if {[info exists :type]} {
-	  switch -glob ${:type} {
-	    "boolean" -
-	    "integer" {set :mongotype ${:type}}
-	    "embedded" {set :mongotype embedded_object}
-	    "reference" {set :mongotype referenced_object}
-	  }
-	  #"::*" {set :mongotype object}
-	}
+        set :mongotype string
+        if {[info exists :type]} {
+          switch -glob ${:type} {
+            "boolean" -
+            "integer" {set :mongotype ${:type}}
+            "embedded" {set :mongotype embedded_object}
+            "reference" {set :mongotype referenced_object}
+          }
+          #"::*" {set :mongotype object}
+        }
       }
       #puts stderr "mongo type of ${:name} is ${:mongotype} [info exists :type]"
       next
     }
-    
+
     #
     # The methods "bson encode|decode" perform the low level type
     # mapping. For now, this handles just the array notation.
     #
-    :method "bson decode" {bsontype value} {
+    :public method "bson decode" {bsontype value} {
       #puts stderr "bson decode of ${:name} /$bsontype/ '$value'"
       if {$bsontype eq "array"} {
-	if {![:isMultivalued]} {
-	  # We got an array, but the slot is not multivalued. Maybe
-	  # generating an error is too harsh, but for the mapping back,
-	  # we check for multivalued as well.
+        if {![:isMultivalued]} {
+          # We got an array, but the slot is not multivalued. Maybe
+          # generating an error is too harsh, but for the mapping back,
+          # we check for multivalued as well.
 
-	  return -code error "Attribute ${:name} should be multivalued, but it is not"
-	}
-	set result [list]
-	foreach {pos type v} $value {lappend result [:bson decode $type $v]}
-	return $result
+          return -code error "Attribute ${:name} should be multivalued, but it is not"
+        }
+        set result [list]
+        foreach {pos type v} $value {lappend result [:bson decode $type $v]}
+        return $result
       } elseif {$bsontype eq "document"} {
-	#puts stderr "*** we have an document '$value', [:serialize]"
-	if {${:type} eq "embedded" && [info exists :arg]} {
-	  #puts stderr "*** we have an embed class = ${:arg}"
-	  set value [${:arg} bson create $value]
-	  #puts stderr "*** ${:arg} bson create ==> $value"
-	} elseif {${:type} eq "reference" && [info exists :arg]} {
-	  #puts stderr "*** we have a reference, class = ${:arg}"
-	  # TODO we assume auto_deref
-	  set value [:bson deref ${:arg} $value]
-	  #puts stderr "*** bson deref ${:arg} ==> $value"
-	} else {
-	  return -code error "don't know how to decode document with value '$value'; [:serialize]"
-	}
+        #puts stderr "*** we have an document '$value', [:serialize]"
+        if {${:type} eq "embedded" && [info exists :arg]} {
+          #puts stderr "*** we have an embedded class = ${:arg}"
+          set value [${:arg} bson create $value]
+          #puts stderr "*** ${:arg} bson create ==> $value"
+        } elseif {${:type} eq "reference" && [info exists :arg]} {
+          #puts stderr "*** we have a reference, class = ${:arg}"
+          # TODO we assume auto_deref
+          set value [:bson deref ${:arg} $value]
+          #puts stderr "*** bson deref ${:arg} ==> $value"
+        } else {
+          return -code error "don't know how to decode document with value '$value'; [:serialize]"
+        }
       }
       return $value
     }
@@ -263,20 +260,20 @@ namespace eval ::nx::mongo {
     :method "bson deref" {class value} {
       #puts stderr "*** bson deref $class '$value'"
       foreach {name type v} $value {
-	if {[string match {$*} $name]} {set ([string range $name 1 end]) $v}
+        if {[string match {$*} $name]} {set ([string range $name 1 end]) $v}
       }
       if {![info exists (id)]} {
-	return -code error "value to be dereferenced does not contain dbref id: $value"
+        return -code error "value to be dereferenced does not contain dbref id: $value"
       }
       if {[info exists (db)]} {
-	if {$(db) ne [$class cget -mongo_db]} {
-	  return -code error "$(db) is different to [$class cget -mongo_db]"
-	}
+        if {$(db) ne [$class cget -mongo_db]} {
+          return -code error "$(db) is different to [$class cget -mongo_db]"
+        }
       }
       if {[info exists (ref)]} {
-	if {$(ref) ne [$class cget -mongo_collection]} {
-	  return -code error "$(ref) is different to [$class cget -mongo_collection]"
-	}
+        if {$(ref) ne [$class cget -mongo_collection]} {
+          return -code error "$(ref) is different to [$class cget -mongo_collection]"
+        }
       }
       return [$class find first -cond [list _id = $(id)]]
     }
@@ -284,20 +281,20 @@ namespace eval ::nx::mongo {
     :method "bson encodeValue" {value} {
       if {${:mongotype} eq "embedded_object"} {
         #puts "embedded_object <$value>"
-	return [list document [$value bson encode]]
+        return [list document [$value bson encode]]
       } elseif {${:mongotype} eq "referenced_object"} {
-	if {![::nsf::var::exists $value _id]} {
-	  :log "autosave $value to obtain an object_id"
-	  $value save
-	}
-	set _id [$value cget -_id]
-	set cls [$value info class]
-	return [list document [list \
-				 {$ref} string [$cls cget -mongo_collection] \
-				 {$id} oid $_id \
-				 {$db} string [$cls cget -mongo_db]]]
+        if {![::nsf::var::exists $value _id]} {
+          :log "autosave $value to obtain an object_id"
+          $value save
+        }
+        set _id [$value cget -_id]
+        set cls [$value info class]
+        return [list document [list \
+                                   {$ref} string [$cls cget -mongo_collection] \
+                                   {$id} oid $_id \
+                                   {$db} string [$cls cget -mongo_db]]]
       } else {
-	return [list ${:mongotype} $value]
+        return [list ${:mongotype} $value]
       }
     }
 
@@ -310,23 +307,23 @@ namespace eval ::nx::mongo {
 
     :public method "bson encode" {-array:switch value} {
       if {[:isMultivalued] || $array} {
-	return [:bson encodeArray $value]
+        return [:bson encodeArray $value]
       } else {
-	return [:bson encodeValue $value]
+        return [:bson encodeValue $value]
       }
     }
 
     :public method remove {object value} {
       if {[:isMultivalued]} {
-	set values [::nsf::var::set $object ${:name}]
-	set p [lsearch $values $value]
-	if {$p < 0} {
-	  return -code error "$value not included in $object.$value ($values)"
-	}
-	set newValues [lreplace $values $p $p]
-	::nsf::var::set $object ${:name} $newValues
+        set values [::nsf::var::set $object ${:name}]
+        set p [lsearch $values $value]
+        if {$p < 0} {
+          return -code error "$value not included in $object.$value ($values)"
+        }
+        set newValues [lreplace $values $p $p]
+        ::nsf::var::set $object ${:name} $newValues
       } else {
-	return -code error "remove just implemented for multivalued slots"
+        return -code error "remove just implemented for multivalued slots"
       }
     }
 
@@ -339,7 +336,7 @@ namespace eval ::nx::mongo {
       # might be object itself or a variable slot managing the object.
       set s [:uplevel self]
       #puts stderr "XXXX check $name '$value' arg='$arg' s=$s // [:uplevel 1 self]"
-      # The calling objects have the the mongo::Object mixed in.
+      # The calling objects have the mongo::Object mixed in.
       if {![$s info has mixin ::nx::mongo::Object]} {
         # If this is not the case, we might be in a variable slot,
         # where we cannot trust the incoming name and we have to
@@ -358,10 +355,10 @@ namespace eval ::nx::mongo {
       }
 
       if {[::nsf::object::exists $value] && [::nsf::is class $arg] && [$value info has type $arg]} {
-	::nsf::var::set $value __embedded_in [list $s $name]
-	::nsf::var::set $s __contains($value) 1
+        ::nsf::var::set $value __embedded_in [list $s $name]
+        ::nsf::var::set $s __contains($value) 1
       } else {
-	return -code error "value '$value' for property $name is not of type $arg"
+        return -code error "value '$value' for property $name is not of type $arg"
       }
     }
     #
@@ -372,16 +369,16 @@ namespace eval ::nx::mongo {
       set s [:uplevel self]
       #puts stderr "check $name '$value' arg='$arg' s=$s"
       if {[::nsf::object::exists $value] && [::nsf::is class $arg] && [$value info has type $arg]} {
-	set ref [list $s $name]
-	if {[::nsf::var::exists $value __referenced_in]} {
-	  set refs [::nsf::var::set $value __referenced_in]
-	  if {[lsearch $refs $ref] == -1} {lappend refs $ref}
-	} else {
-	  set refs [list $ref]
-	}
-	::nsf::var::set $value __referenced_in $refs
+        set ref [list $s $name]
+        if {[::nsf::var::exists $value __referenced_in]} {
+          set refs [::nsf::var::set $value __referenced_in]
+          if {[lsearch $refs $ref] == -1} {lappend refs $ref}
+        } else {
+          set refs [list $ref]
+        }
+        ::nsf::var::set $value __referenced_in $refs
       } else {
-	return -code error "value '$value' for property $name is not of type $arg"
+        return -code error "value '$value' for property $name is not of type $arg"
       }
     }
     #
@@ -395,14 +392,14 @@ namespace eval ::nx::mongo {
       return [expr {[clock scan $value] * 1000}]
     }
   }
-  
+
 
   #######################################################################
   # The class mongo::Class provides methods for mongo classes (such as
   # "find", "insert", ...)
   #
   ::nx::Class create ::nx::mongo::Class -superclass nx::Class {
-    
+
     #
     # Every mongo class can be configured with a mongo_ns, from which
     # its instance data is queried.
@@ -410,27 +407,28 @@ namespace eval ::nx::mongo {
     :property mongo_ns
     :property mongo_db
     :property mongo_collection
-    
+
     #
     # Provide helper methods to access from an external specifier
     # (property name or operator name) internal representations
-    # (eg. mongo type, or mongo operator).
+    # (e.g. mongo type, or mongo operator).
     #
-    :method "get slot" {att} {
-      set classes [concat [self] [:info mixins] [:info heritage]]
+    :public method "get slot" {att} {
+      set classes [list [self] {*}[:info mixins] {*}[:info heritage]]
+      #puts stderr "searching for <$att> along <$classes>"
       foreach cls $classes {
-	set slot [$cls info slots $att]
-	if {$slot ne ""} {
-	  return $slot
-	}
+        set slot [$cls info slots $att]
+        if {$slot ne ""} {
+          return $slot
+        }
       }
     }
-    
+
     :public method "get relop" {op} {
       array set "" {< $lt > $gt <= $lte >= $gte != $ne in $in all $all}
       return $($op)
     }
-    
+
     #
     # For interaction with bson structures, we provide on the class
     # level "bson cond" (a small dsl for a more convenient syntax in
@@ -444,23 +442,28 @@ namespace eval ::nx::mongo {
       #puts "bson cond $cond"
       set bson [list]
       foreach {att op value} $cond {
-	set slot [:get slot $att]
-	if {$slot eq ""} {return -code error "could not obtain slot for <$att $op $value>"}
-	switch $op {
-	  "=" {lappend bson $att [$slot cget -mongotype] $value}
-	  ">" - "<" - "<=" - ">=" - "!="  {
-	    lappend bson $att document [list [:get relop $op] [$slot cget -mongotype] $value]
-	  }
-	  "in" - "all" {
-	    lappend bson $att document [list [:get relop $op] {*}[$slot bson encode -array $value]]
-	  }
-	  default {return -code error "unknown operator $op"}
-	}
+        set slot [:get slot $att]
+        if {$slot eq ""} {return -code error "could not obtain slot for <$att $op $value>"}
+        switch $op {
+          "=" {lappend bson $att [$slot cget -mongotype] $value}
+          ">" - "<" - "<=" - ">=" - "!="  {
+            lappend bson $att document [list [:get relop $op] [$slot cget -mongotype] $value]
+          }
+          "in" - "all" {
+            lappend bson $att document [list [:get relop $op] {*}[$slot bson encode -array $value]]
+          }
+          "~" {
+            # value should be a two-element list contain pattern and options
+            lappend bson $att document [list {$regex} regex $value]
+          }
+
+          default {return -code error "unknown operator $op"}
+        }
       }
       #puts "bson cond <$cond> => $bson"
       return $bson
     }
-    
+
     :method "bson opts" {{-orderby ""} {-atts ""} -limit:integer -skip:integer} {
       set result ""
       if {$atts ne ""} {
@@ -486,14 +489,14 @@ namespace eval ::nx::mongo {
       }
       return $bson
     }
-    
+
     :method "bson atts" {atts} {
       set result {}
       foreach {att value} $atts {
-	if {![string is integer -strict $value]} {
-	  return -code error "$atts: $value should be integer"
-	}
-	lappend result $att int $value
+        if {![string is integer -strict $value]} {
+          return -code error "$atts: $value should be integer"
+        }
+        lappend result $att int $value
       }
       return $result
     }
@@ -506,15 +509,15 @@ namespace eval ::nx::mongo {
       #puts "bson parameter: <$tuple>"
       set objParams [list]
       foreach {att type value} $tuple {
-	set slot [:get slot $att]
-	if {$slot eq ""} {return -code error "could not obtain slot for <$att $type $value>"}
-	lappend objParams -$att [$slot bson decode $type $value]
+        set slot [:get slot $att]
+        if {$slot eq ""} {return -code error "could not obtain slot for <$att $type $value>"}
+        lappend objParams -$att [$slot bson decode $type $value]
       }
       #puts "bson parameter <$tuple> => $objParams"
       return $objParams
     }
 
-    :method "bson setvalues" {tuple} {
+    :public method "bson setvalues" {tuple} {
       #
       # Translate bson tuple into a cmd to set instance values, which
       # can be evaluated in the context of an object.
@@ -522,8 +525,8 @@ namespace eval ::nx::mongo {
       #puts "bson setvalues: <$tuple>"
       set cmd ""
       foreach {att type value} $tuple {
-	set slot [:get slot $att]
-	if {$slot eq ""} {return -code error "could not obtain slot for <$att $type $value>"}
+        set slot [:get slot $att]
+        if {$slot eq ""} {return -code error "could not obtain slot for <$att $type $value>"}
         if {[nx::var exists $slot rep] && [nx::var set $slot rep] ne ""} {
           set script [:bson rep decode [nx::var set $slot rep] $slot $att $type $value]
           append cmd $script\n
@@ -536,17 +539,35 @@ namespace eval ::nx::mongo {
     }
 
     :public method "bson create" {{-name ""} tuple} {
-      ::nsf::object::alloc [self] $name [:bson setvalues $tuple]
+      #puts stderr "=== [self] bson create name <$name> tuple <$tuple>"
+      set class [self]
+      set filtered_tuple {}
+      foreach {att_name att_type att_value} $tuple {
+        if {$att_name eq "__class"} {
+          #
+          # Special handlnig of "__class" attribute
+          #
+          if {[nsf::is class $att_value]} {
+            set class $att_value
+          }
+          continue
+        }
+        lappend filtered_tuple $att_name $att_type $att_value
+      }
+      #puts "FINAL class $class, values\n[$class bson setvalues $filtered_tuple]"
+      set o [::nsf::object::alloc $class $name [$class bson setvalues $filtered_tuple]]
+      $o eval :init
+      return $o
     }
 
     :method "bson pp_array" {{-indent 0} list} {
       set result [list]
       foreach {name type value} $list {
-	switch $type {
-	  document { lappend result "\{ [:bson pp -indent $indent $value] \}" }
-	  array    { lappend result "\[ [:bson pp_array -indent $indent $value] \]" }
-	  default  { lappend result [list $value]}
-	}
+        switch $type {
+          document { lappend result "\{ [:bson pp -indent $indent $value] \}" }
+          array    { lappend result "\[ [:bson pp_array -indent $indent $value] \]" }
+          default  { lappend result [list $value]}
+        }
       }
       return [join $result ", "]
     }
@@ -555,12 +576,12 @@ namespace eval ::nx::mongo {
       set result [list]
       set nextIndent [expr {$indent + 2}]
       foreach {name type value} $list {
-	set prefix "\n[string repeat { } $indent]$name: "
-	switch $type {
-	  document { lappend result "$prefix\{ [:bson pp -indent $nextIndent $value] \}" }
-	  array    { lappend result "$prefix\[ [:bson pp_array -indent $nextIndent $value] \]" }
-	  default  { lappend result $prefix[list $value]}
-	}
+        set prefix "\n[string repeat { } $indent]$name: "
+        switch $type {
+          document { lappend result "$prefix\{ [:bson pp -indent $nextIndent $value] \}" }
+          array    { lappend result "$prefix\[ [:bson pp_array -indent $nextIndent $value] \]" }
+          default  { lappend result $prefix[list $value]}
+        }
       }
       return [join $result ", "]
     }
@@ -570,39 +591,41 @@ namespace eval ::nx::mongo {
     # default slot class
     #
     :public method property {
-                             {-accessor ""}
-                             {-class ::nx::mongo::Attribute}
-                             {-configurable:boolean true}
-                             {-incremental:switch}
-                             {-rep ""}
-                             spec:parameter
-                             {initblock ""}
-                           } {
+      {-accessor ""}
+      {-class ::nx::mongo::Attribute}
+      {-configurable:boolean true}
+      {-incremental:switch}
+      {-rep ""}
+      spec:parameter
+      {initblock ""}
+    } {
       regsub -all {,type=::} $spec {,arg=::} spec
       set result [next [list -accessor $accessor -class $class \
                             -configurable $configurable -incremental=$incremental \
                             $spec $initblock]]
-      lassign [::nx::MetaSlot parseParameterSpec $spec] name 
+      lassign [::nx::MetaSlot parseParameterSpec -target [self] {*}$spec] name parameterOptions class options
+      #puts stderr "==== spec <$spec> => name <$name> parameterOptions <$parameterOptions> class <$class> options <$options>"
+      #puts stderr "==== [list [self] property [:info slots $name] configure -rep $rep]"
       [:info slots $name] configure -rep $rep
       return $result
     }
 
     :public method variable {
-                             {-accessor "none"}
-                             {-class ::nx::mongo::Attribute}
-                             {-configurable:boolean false}
-                             {-incremental:switch}
-                             {-initblock ""}
-                             {-rep ""}
-                             spec:parameter
-                             defaultValue:optional
-                           } {
+      {-accessor "none"}
+      {-class ::nx::mongo::Attribute}
+      {-configurable:boolean false}
+      {-incremental:switch}
+      {-initblock ""}
+      {-rep ""}
+      spec:parameter
+      defaultValue:optional
+    } {
       regsub -all {,type=::} $spec {,arg=::} spec
       set result [next [list -accessor $accessor -class $class \
                             -configurable $configurable -incremental=$incremental \
                             -initblock $initblock $spec \
                             {*}[expr {[info exists defaultValue] ? [list $defaultValue] : ""}]]]
-      lassign [::nx::MetaSlot parseParameterSpec $spec] name 
+      lassign [::nx::MetaSlot parseParameterSpec -target [self] $spec] name
       [:info slots $name] configure -rep $rep
       return $result
     }
@@ -630,7 +653,7 @@ namespace eval ::nx::mongo {
       }
       return $vars
     }
-    
+
     #
     # index method
     #
@@ -640,7 +663,7 @@ namespace eval ::nx::mongo {
       # todo: multi-property indices
       db index ${:mongo_ns} [list $att int $type] {*}$args
     }
-    
+
     #
     # A convenience method for inserting a fresh tuple
     #
@@ -659,19 +682,19 @@ namespace eval ::nx::mongo {
     :public method count {{-cond ""}} {
       return [::nx::mongo::db count ${:mongo_ns} [:bson cond $cond]]
     }
-    
+
     #
     # The query interface consists currently of "find first" (returning
     # a single instance) and "find all" (returning a list of instances).
     #
     :public method "find first" {
-				 {-instance ""}
-				 {-atts ""}
-				 {-cond ""}
-				 {-orderby ""} 
-			       } {
+      {-instance ""}
+      {-atts ""}
+      {-cond ""}
+      {-orderby ""}
+    } {
       set tuple [lindex [::nx::mongo::db query ${:mongo_ns} \
-			     [:bson cond $cond] \
+                             [:bson cond $cond] \
                              -opts [:bson opts -atts $atts -limit 1 -orderby $orderby] \
                             ] 0]
       if {$tuple eq ""} {
@@ -680,68 +703,68 @@ namespace eval ::nx::mongo {
       if {$instance ne ""} {set instance [:uplevel [list ::nsf::object::qualify $instance]]}
       return [:bson create -name $instance $tuple]
     }
-    
+
     :public method "find all" {
-			       {-atts ""}
-			       {-cond ""} 
-			       {-orderby ""} 
-			       {-limit:integer} 
-			       {-skip:integer} 
-			     } {
+      {-atts ""}
+      {-cond ""}
+      {-orderby ""}
+      {-limit:integer}
+      {-skip:integer}
+    } {
       set result [list]
       set opts [list]
       if {[info exists limit]} {lappend opts -limit $limit}
       if {[info exists skip]} {lappend opts -skip $skip}
       set fetched [::nx::mongo::db query ${:mongo_ns} \
-		       [:bson cond $cond] \
+                       [:bson cond $cond] \
                        -opts [:bson opts -orderby $orderby -atts $atts {*}$opts] ]
 
       foreach tuple $fetched {
-	lappend result [:bson create $tuple]
+        lappend result [:bson create $tuple]
       }
       return $result
     }
 
     :public method show {
-			 {-atts ""}
-			 {-cond ""} 
-			 {-orderby ""} 
-			 {-limit} 
-			 {-skip} 
-			 {-puts:boolean 1} 
-                       } {
+      {-atts ""}
+      {-cond ""}
+      {-orderby ""}
+      {-limit}
+      {-skip}
+      {-puts:boolean 1}
+    } {
       set opts [list]
       if {[info exists limit]} {lappend opts -limit $limit}
       if {[info exists skip]} {lappend opts -skip $skip}
       set fetched [::nx::mongo::db query ${:mongo_ns} \
-		       [:bson cond $cond] \
+                       [:bson cond $cond] \
                        -opts [:bson opts -orderby $orderby -atts $atts {*}$opts] ]
       set tuples [list]
       foreach tuple $fetched {
-	lappend tuples "\{[:bson pp -indent 4 $tuple]\n\}"
+        lappend tuples "\{[:bson pp -indent 4 $tuple]\n\}"
       }
       if {$puts} {puts [join $tuples ", "]}
       return $tuples
     }
-    
+
     :method mongo_setup {} {
       #
       # setup mongo_collection, mongo_db and mongo_ns
       #
       if {[info exists :mongo_ns]} {
-	#puts stderr "given mongo_ns ${:mongo_ns}"
-	if {![regexp {^([^.]+)[.](.*)$} ${:mongo_ns} :mongo_db :mongo_collection]} {
-	  return -code error "${:mongo_ns} does not contain a dot."
-	}
+        #puts stderr "given mongo_ns ${:mongo_ns}"
+        if {![regexp {^([^.]+)[.](.*)$} ${:mongo_ns} :mongo_db :mongo_collection]} {
+          return -code error "${:mongo_ns} does not contain a dot."
+        }
       } else {
-	if {![info exists :mongo_collection]} {
-	  set :mongo_collection [string tolower [namespace tail [self]]]s
-	}
-	if {![info exists :mongo_db]} {
-	  set :mongo_db [::nx::mongo::db cget -db]
-	}
-	set :mongo_ns ${:mongo_db}.${:mongo_collection}
-	#puts stderr "mongo_ns is set to ${:mongo_ns}"
+        if {![info exists :mongo_collection]} {
+          set :mongo_collection [string tolower [namespace tail [self]]]s
+        }
+        if {![info exists :mongo_db]} {
+          set :mongo_db [::nx::mongo::db cget -db]
+        }
+        set :mongo_ns ${:mongo_db}.${:mongo_collection}
+        #puts stderr "mongo_ns is set to ${:mongo_ns}"
       }
     }
 
@@ -749,7 +772,7 @@ namespace eval ::nx::mongo {
     # When a mongo::Class is created, mixin the mongo::Object to make
     # "save" etc. available
     #
-    
+
     :method init {} {
       :mixins add ::nx::mongo::Object
       :mongo_setup
@@ -777,7 +800,7 @@ namespace eval ::nx::mongo {
   # variables and for decode an setting these. The codecs
   # (coder/decoder) are extensible on the application level by
   # defining ensemble methods with the name of the codec as last part.
-  
+
   ::nx::mongo::Class eval {
     #
     # rep codec "array"
@@ -793,7 +816,7 @@ namespace eval ::nx::mongo {
     :public method "bson rep decode array" {slot name bsontype value} {
       set av ""
       foreach {pos type entry} $value {
-        lappend av [lindex $entry 2] [lindex $entry 5] 
+        lappend av [lindex $entry 2] [lindex $entry 5]
       }
       return "array set :$name [list $av]"
     }
@@ -804,7 +827,7 @@ namespace eval ::nx::mongo {
     :public method "bson rep encode dict" {slot obj name} {
       set body {}
       dict for {k v} [$obj eval [list set :$name]] {
-        lappend body $k string $v 
+        lappend body $k string $v
       }
       return [list document $body]
     }
@@ -816,22 +839,37 @@ namespace eval ::nx::mongo {
       return "set :$name \[dict create $result\]"
     }
   }
-  
+
   #######################################################################
   # The class mongo::Object provides methods for mongo objects (such as
   # "save")
   #
   ::nx::Class create ::nx::mongo::Object {
-    
+
     #
     # manage logging of mongo concerns
     #
     :public method log {msg} {
       if {$::nx::mongo::log} {
-	nsf::log notice "mongo: $msg"
+        nsf::log notice "mongo: $msg"
       }
     }
 
+    #
+    # Manage lifecycle concerns for NaviServer/AOLserver: at the end
+    # of every request, all global variables are deleted. We can use
+    # this to destroy as well volatile mongoDB objects via Tcl
+    # variable traces.
+    #
+    :public method destroy_on_cleanup {} {
+      set name ::__volatile_mongo_objects([self])
+      set $name 1
+      trace add variable $name unset [list [self] __cleanup]
+    }
+    :public method __cleanup {args} {
+      :destroy
+    }
+
     #
     # _id is the special property maintained by mongoDB
     #
@@ -842,19 +880,20 @@ namespace eval ::nx::mongo {
     #
     # Encode all object data in bson notation
     #
-    :method "bson encode" {{-ignore ""}} {
+    :public method "bson encode" {{-ignore ""}} {
       set bson [list]
       set cls [:info class]
+      lappend bson "__class" string $cls
       foreach var [:info vars] {
         if {$var in $ignore} continue
-	set slot [$cls get slot $var]
-	if {$slot ne ""} {
+        set slot [$cls get slot $var]
+        if {$slot ne ""} {
           if {[nx::var exists $slot rep] && [nx::var set $slot rep] ne ""} {
             lappend bson $var {*}[$cls bson rep encode [nx::var set $slot rep] $slot [self] $var]
           } else {
             lappend bson $var {*}[$slot bson encode [set :$var]]
           }
-	}
+        }
       }
       return $bson
     }
@@ -864,15 +903,15 @@ namespace eval ::nx::mongo {
     #
     :public method destroy {} {
       if {[array exists :__contains]} {
-	# destroy embedded object
-	foreach o [array names :__contains] {
-	  :log "[self] contains $o -> destroy"
-	  $o destroy
-	}
+        # destroy embedded object
+        foreach o [array names :__contains] {
+          :log "[self] contains $o -> destroy"
+          $o destroy
+        }
       }
       if {[info exists :__embedded_in]} {
-	lassign ${:__embedded_in} parent att
-	::nsf::var::unset $parent __contains([self])
+        lassign ${:__embedded_in} parent att
+        ::nsf::var::unset $parent __contains([self])
       }
       next
     }
@@ -883,45 +922,45 @@ namespace eval ::nx::mongo {
     :public method delete {} {
       if {[info exists :__embedded_in]} {
         #
-	# When an embedded object is deleted, it is removed for the
-	# reference list. The containing object is not automatically
-	# saved for the time being. We could consider an automatic
-	# save or mongo-$pull update operation.
+        # When an embedded object is deleted, it is removed for the
+        # reference list. The containing object is not automatically
+        # saved for the time being. We could consider an automatic
+        # save or mongo-$pull update operation.
         #
-	#puts "[self] is embedded in ${:__embedded_in}"
-	lassign ${:__embedded_in} parent att
-	set slot [[$parent info class] get slot $att]
-	if {$slot eq ""} {return -code error "could not obtain slot for <[$parent info class] $att>"}
-	$slot remove $parent [self]
-	#puts stderr [:serialize]
-	:log "[self] must save parent $parent in db"
-	:destroy
+        #puts "[self] is embedded in ${:__embedded_in}"
+        lassign ${:__embedded_in} parent att
+        set slot [[$parent info class] get slot $att]
+        if {$slot eq ""} {return -code error "could not obtain slot for <[$parent info class] $att>"}
+        $slot remove $parent [self]
+        #puts stderr [:serialize]
+        :log "[self] must save parent $parent in db"
+        :destroy
       } elseif {[info exists :__referenced_in]} {
         #
-	# When a referenced is deleted, we do for now essentially the
-	# same as for embedded objects. However, the same object might
-	# be referenced by several objects.
+        # When a referenced is deleted, we do for now essentially the
+        # same as for embedded objects. However, the same object might
+        # be referenced by several objects.
         #
-	#puts "[self] is referenced in ${:__referenced_in}"
-	foreach reference ${:__referenced_in} {
-	  lassign $reference parent att
-	  set slot [[$parent info class] get slot $att]
-	  if {$slot eq ""} {return -code error "could not obtain slot for <[$parent info class] $att>"}
-	  $slot remove $parent [self]
-	  :log "[self] must save parent $parent in db"
-	}
-	:destroy
+        #puts "[self] is referenced in ${:__referenced_in}"
+        foreach reference ${:__referenced_in} {
+          lassign $reference parent att
+          set slot [[$parent info class] get slot $att]
+          if {$slot eq ""} {return -code error "could not obtain slot for <[$parent info class] $att>"}
+          $slot remove $parent [self]
+          :log "[self] must save parent $parent in db"
+        }
+        :destroy
       } else {
-	#puts "delete a non-embedded entry"
-	if {[info exists :_id]} {
-	  set mongo_ns [[:info class] cget -mongo_ns]
-	  ::nx::mongo::db delete $mongo_ns [list _id oid ${:_id}]
-	} else {
-	  return -code error "[self]: object does not contain an _id; it can't be delete from the mongo db."
-	}
+        #puts "delete a non-embedded entry"
+        if {[info exists :_id]} {
+          set mongo_ns [[:info class] cget -mongo_ns]
+          ::nx::mongo::db delete $mongo_ns [list _id oid ${:_id}]
+        } else {
+          return -code error "[self]: object does not contain an _id; it can't be delete from the mongo db."
+        }
       }
     }
-    
+
     #
     # Save the current object. When we have an _id, perform an update,
     # otherwise perform an insert
@@ -929,26 +968,26 @@ namespace eval ::nx::mongo {
     :public method save {} {
       set mongo_ns [[:info class] cget -mongo_ns]
       if {$mongo_ns eq ""} {
-	# We could perform the delegation probably automatically, but
-	# for now we provide an error
-	return -code error "No mongo_ns specified for [:info class]. In case this is an embedded object, save the embedding one."
+        # We could perform the delegation probably automatically, but
+        # for now we provide an error
+        return -code error "No mongo_ns specified for [:info class]. In case this is an embedded object, save the embedding one."
       } else {
-	set bson [:bson encode]
-	if {[info exists :_id]} {
-	  :log "we have to update [[:info class] bson pp -indent 4 $bson]"
-	  ::nx::mongo::db update $mongo_ns [list _id oid ${:_id}] $bson
-	  set :_id
-	} else {
-	  :log "we have to insert [[:info class] bson pp -indent 4 $bson]"
-	  set r [::nx::mongo::db insert $mongo_ns $bson]
-	  set :_id [lindex $r 2]
-	}
+        set bson [:bson encode]
+        if {[info exists :_id]} {
+          :log "we have to update [[:info class] bson pp -indent 4 $bson]"
+          ::nx::mongo::db update $mongo_ns [list _id oid ${:_id}] $bson
+          set :_id
+        } else {
+          :log "we have to insert [[:info class] bson pp -indent 4 $bson]"
+          set r [::nx::mongo::db insert $mongo_ns $bson]
+          set :_id [lindex $r 2]
+        }
       }
     }
   }
-  
-}
 
+}
+#puts stderr "NX MONGO LOADED"
 #
 # Local variables:
 #    mode: tcl
